@@ -13,8 +13,9 @@
  *-------
  */
 
-#include <stdlib.h>
+/* #include <stdlib.h> */
 #include <string.h>
+#include <ctype.h>
 #include "bind.h"
 
 #include "environ.h"
@@ -132,9 +133,9 @@ PGAPI_BindParameter(
 		SC_recycle_statement(stmt);
 
 	mylog("%s: ipar=%d, paramType=%d, fCType=%d, fSqlType=%d, cbColDef=%d, ibScale=%d,", func, ipar, fParamType, fCType, fSqlType, cbColDef, ibScale);
-	/*** mylog("rgbValue=%x, pcbValue = %x(%d), data_at_exec = %d\n", rgbValue, pcbValue, pcbValue ? *pcbValue : -777, apdopts->parameters[ipar].data_at_exec); ***/
+	/*** mylog("rgbValue=%p, pcbValue = %p(%d), data_at_exec = %d\n", rgbValue, pcbValue, pcbValue ? *pcbValue : -777, apdopts->parameters[ipar].data_at_exec); ***/
 
-	mylog("rgbValue=%x, pcbValue = %x, data_at_exec = %d\n", rgbValue, pcbValue, apdopts->parameters[ipar].data_at_exec);
+	mylog("rgbValue=%p, pcbValue = %p, data_at_exec = %d\n", rgbValue, pcbValue, apdopts->parameters[ipar].data_at_exec);
 
 	return SQL_SUCCESS;
 }
@@ -159,8 +160,8 @@ PGAPI_BindCol(
 
 	mylog("%s: entering...\n", func);
 
-	mylog("**** PGAPI_BindCol: stmt = %x, icol = %d\n", stmt, icol);
-	mylog("**** : fCType=%d rgb=%x valusMax=%d pcb=%x\n", fCType, rgbValue, cbValueMax, pcbValue);
+	mylog("**** PGAPI_BindCol: stmt = %p, icol = %d\n", stmt, icol);
+	mylog("**** : fCType=%d rgb=%p valusMax=%d pcb=%p\n", fCType, rgbValue, cbValueMax, pcbValue);
 
 	if (!stmt)
 	{
@@ -270,7 +271,7 @@ inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
 			opts->bindings[icol].precision = 0;
 		opts->bindings[icol].scale = 0;
 
-		mylog("       bound buffer[%d] = %x\n", icol, opts->bindings[icol].buffer);
+		mylog("       bound buffer[%d] = %p\n", icol, opts->bindings[icol].buffer);
 	}
 
 cleanup:
@@ -337,12 +338,13 @@ inolog("num_params=%d\n", stmt->num_params);
 	/* StartRollbackState(stmt); */
 	if (NOT_YET_PREPARED == stmt->prepared)
 	{
-		decideHowToPrepare(stmt);
+		decideHowToPrepare(stmt, FALSE);
 inolog("howTo=%d\n", SC_get_prepare_method(stmt));
 		switch (SC_get_prepare_method(stmt))
 		{
-			case USING_PARSE_REQUEST:
-			case USING_UNNAMED_PARSE_REQUEST:
+			case NAMED_PARSE_REQUEST:
+			case PARSE_TO_EXEC_ONCE:
+			case PARSE_REQ_FOR_INFO:
 				if (ret = prepareParameters(stmt), SQL_ERROR == ret)
 					goto cleanup;
 		}
@@ -392,6 +394,7 @@ cleanup:
 }
 
 
+#if (ODBCVER < 0x0300)
 /*	Sets multiple values (arrays) for the set of parameter markers. */
 RETCODE		SQL_API
 PGAPI_ParamOptions(
@@ -402,14 +405,17 @@ PGAPI_ParamOptions(
 	CSTR func = "PGAPI_ParamOptions";
 	StatementClass *stmt = (StatementClass *) hstmt;
 	APDFields	*apdopts;
+	IPDFields	*ipdopts;
 
-	mylog("%s: entering... %d %x\n", func, crow, pirow);
+	mylog("%s: entering... %d %p\n", func, crow, pirow);
 
 	apdopts = SC_get_APDF(stmt);
 	apdopts->paramset_size = crow;
-	SC_get_IPDF(stmt)->param_processed_ptr = (UInt4 *) pirow;
+	ipdopts = SC_get_IPDF(stmt);
+	ipdopts->param_processed_ptr = pirow;
 	return SQL_SUCCESS;
 }
+#endif /* ODBCVER */
 
 
 /*
@@ -458,8 +464,8 @@ inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 	{
 		const	char *sptr, *tag = NULL;
 		ConnectionClass	*conn = SC_get_conn(stmt);
-		int	taglen;
-		char	tchar, bchar, escape_in_literal;
+		size_t	taglen = 0;
+		char	tchar, bchar, escape_in_literal = '\0';
 		char	in_literal = FALSE, in_identifier = FALSE,
 			in_dollar_quote = FALSE, in_escape = FALSE,
 			multi = FALSE, del_found = FALSE;
@@ -526,7 +532,7 @@ inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 					in_dollar_quote = TRUE;
 					tag = sptr;
 					taglen = 0; 
-					if (dollar_next = strchr(sptr + 1, dollar_quote))
+					if (dollar_next = strchr(sptr + 1, dollar_quote), NULL != dollar_next)
 					{
 						taglen = dollar_next - sptr + 1;
 						sptr = dollar_next;
@@ -586,7 +592,7 @@ extend_parameter_bindings(APDFields *self, int num_params)
 	CSTR func = "extend_parameter_bindings";
 	ParameterInfoClass *new_bindings;
 
-	mylog("%s: entering ... self=%x, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%p, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -619,7 +625,7 @@ extend_iparameter_bindings(IPDFields *self, int num_params)
 	CSTR func = "extend_iparameter_bindings";
 	ParameterImplClass *new_bindings;
 
-	mylog("%s: entering ... self=%x, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%p, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -651,7 +657,7 @@ reset_a_parameter_binding(APDFields *self, int ipar)
 {
 	CSTR func = "reset_a_parameter_binding";
 
-	mylog("%s: entering ... self=%x, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
+	mylog("%s: entering ... self=%p, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
 
 	if (ipar < 1 || ipar > self->allocated)
 		return;
@@ -671,7 +677,7 @@ reset_a_iparameter_binding(IPDFields *self, int ipar)
 {
 	CSTR func = "reset_a_iparameter_binding";
 
-	mylog("%s: entering ... self=%x, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
+	mylog("%s: entering ... self=%p, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
 
 	if (ipar < 1 || ipar > self->allocated)
 		return;
@@ -757,7 +763,7 @@ PDATA_free_params(PutDataInfo *pdata, char option)
 {
 	int			i;
 
-	mylog("PDATA_free_params:  ENTER, self=%x\n", pdata);
+	mylog("PDATA_free_params:  ENTER, self=%p\n", pdata);
 
 	if (!pdata->pdata)
 		return;
@@ -792,7 +798,7 @@ PDATA_free_params(PutDataInfo *pdata, char option)
 void
 IPD_free_params(IPDFields *ipdopts, char option)
 {
-	mylog("IPD_free_params:  ENTER, self=%x\n", ipdopts);
+	mylog("IPD_free_params:  ENTER, self=%p\n", ipdopts);
 
 	if (!ipdopts->parameters)
 		return;
@@ -813,7 +819,7 @@ extend_column_bindings(ARDFields *self, int num_columns)
 	BindInfoClass *new_bindings;
 	int			i;
 
-	mylog("%s: entering ... self=%x, bindings_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
+	mylog("%s: entering ... self=%p, bindings_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -866,7 +872,7 @@ reset_a_column_binding(ARDFields *self, int icol)
 	CSTR func = "reset_a_column_binding";
 	BindInfoClass	*bookmark;
 
-	mylog("%s: entering ... self=%x, bindings_allocated=%d, icol=%d\n", func, self, self->allocated, icol);
+	mylog("%s: entering ... self=%p, bindings_allocated=%d, icol=%d\n", func, self, self->allocated, icol);
 
 	if (icol > self->allocated)
 		return;
@@ -896,7 +902,7 @@ void	ARD_unbind_cols(ARDFields *self, BOOL freeall)
 {
 	Int2	lf;
 
-inolog("ARD_unbind_cols freeall=%d allocated=%d bindings=%x", freeall, self->allocated, self->bindings);
+inolog("ARD_unbind_cols freeall=%d allocated=%d bindings=%p", freeall, self->allocated, self->bindings);
 	for (lf = 1; lf <= self->allocated; lf++)
 		reset_a_column_binding(self, lf);
 	if (freeall)
@@ -911,7 +917,7 @@ void	GDATA_unbind_cols(GetDataInfo *self, BOOL freeall)
 {
 	Int2	lf;
 
-inolog("GDATA_unbind_cols freeall=%d allocated=%d gdata=%x", freeall, self->allocated, self->gdata);
+inolog("GDATA_unbind_cols freeall=%d allocated=%d gdata=%p", freeall, self->allocated, self->gdata);
 	if (self->fdata.ttlbuf)
 	{
 		free(self->fdata.ttlbuf);
@@ -963,9 +969,8 @@ extend_getdata_info(GetDataInfo *self, int num_columns, BOOL shrink)
 {
 	CSTR func = "extend_getdata_info";
 	GetDataClass	*new_gdata;
-	int			i;
 
-	mylog("%s: entering ... self=%x, gdata_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
+	mylog("%s: entering ... self=%p, gdata_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -988,6 +993,8 @@ extend_getdata_info(GetDataInfo *self, int num_columns, BOOL shrink)
 		}
 		if (self->gdata)
 		{
+			size_t	i;
+
 			for (i = 0; i < self->allocated; i++)
 				new_gdata[i] = self->gdata[i];
 			free(self->gdata);
@@ -997,6 +1004,8 @@ extend_getdata_info(GetDataInfo *self, int num_columns, BOOL shrink)
 	}
 	else if (shrink && self->allocated > num_columns)
 	{
+		int	i;
+
 		for (i = self->allocated; i > num_columns; i--)
 			reset_a_getdata_info(self, i);
 		self->allocated = num_columns;
@@ -1041,7 +1050,7 @@ extend_putdata_info(PutDataInfo *self, int num_params, BOOL shrink)
 	CSTR func = "extend_putdata_info";
 	PutDataClass	*new_pdata;
 
-	mylog("%s: entering ... self=%x, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%p, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
