@@ -42,6 +42,10 @@
 
 #define PRN_NULLCHECK
 
+static void CC_lookup_pg_version(ConnectionClass *self);
+static void CC_lookup_lo(ConnectionClass *self);
+static char *CC_create_errormsg(ConnectionClass *self);
+
 extern GLOBAL_VALUES globals;
 
 
@@ -57,7 +61,7 @@ PGAPI_AllocConnect(
 	mylog("%s: entering...\n", func);
 
 	conn = CC_Constructor();
-	mylog("**** %s: henv = %x, conn = %x\n", func, henv, conn);
+	mylog("**** %s: henv = %p, conn = %p\n", func, henv, conn);
 
 	if (!conn)
 	{
@@ -99,8 +103,9 @@ PGAPI_Connect(
 	ConnInfo   *ci;
 	CSTR func = "PGAPI_Connect";
 	RETCODE	ret = SQL_SUCCESS;
+	char	fchar;
 
-	mylog("%s: entering...\n", func);
+	mylog("%s: entering..cbDSN=%hi.\n", func, cbDSN);
 
 	if (!conn)
 	{
@@ -123,13 +128,19 @@ PGAPI_Connect(
 	 * override values from DSN info with UID and authStr(pwd) This only
 	 * occurs if the values are actually there.
 	 */
+	fchar = ci->username[0]; /* save the first byte */ 
 	make_string(szUID, cbUID, ci->username, sizeof(ci->username));
+	if ('\0' == ci->username[0]) /* an empty string is specified */
+		ci->username[0] = fchar; /* restore the original username */
+	fchar = ci->password[0]; 
 	make_string(szAuthStr, cbAuthStr, ci->password, sizeof(ci->password));
+	if ('\0' == ci->password[0]) /* an empty string is specified */
+		ci->password[0] = fchar; /* restore the original password */
 
 	/* fill in any defaults */
 	getDSNdefaults(ci);
 
-	qlog("conn = %x, %s(DSN='%s', UID='%s', PWD='%s')\n", conn, func, ci->dsn, ci->username, ci->password ? "xxxxx" : "");
+	qlog("conn = %p, %s(DSN='%s', UID='%s', PWD='%s')\n", conn, func, ci->dsn, ci->username, ci->password ? "xxxxx" : "");
 
 	if (CC_connect(conn, AUTH_REQ_OK, NULL) <= 0)
 	{
@@ -180,7 +191,7 @@ PGAPI_Disconnect(
 		return SQL_INVALID_HANDLE;
 	}
 
-	qlog("conn=%x, %s\n", conn, func);
+	qlog("conn=%p, %s\n", conn, func);
 
 	if (conn->status == CONN_EXECUTING)
 	{
@@ -209,7 +220,7 @@ PGAPI_FreeConnect(
 	CSTR func = "PGAPI_FreeConnect";
 
 	mylog("%s: entering...\n", func);
-	mylog("**** in %s: hdbc=%x\n", func, hdbc);
+	mylog("**** in %s: hdbc=%p\n", func, hdbc);
 
 	if (!conn)
 	{
@@ -232,6 +243,31 @@ PGAPI_FreeConnect(
 }
 
 
+static void
+CC_globals_init(GLOBAL_VALUES *globs)
+{
+	memset(globs, 0, sizeof(GLOBAL_VALUES));
+	globs->fetch_max = -1001;
+	globs->socket_buffersize = -1001;
+	globs->unknown_sizes = -1;
+	globs->max_varchar_size = -1001;
+	globs->max_longvarchar_size = -1001;
+
+	globs->debug = -1;
+	globs->commlog = -1;
+	globs->disable_optimizer = -1;
+	globs->ksqo = -1;
+	globs->unique_index = -1;
+	globs->onlyread = -1;
+	globs->use_declarefetch = -1;
+	globs->text_as_longvarchar = -1;
+	globs->unknowns_as_longvarchar = -1;
+	globs->bools_as_char = -1;
+	globs->lie = -1;
+	globs->parse = -1;
+	globs->cancel_as_freestmt = -1;
+}
+
 void
 CC_conninfo_init(ConnInfo *conninfo)
 {
@@ -246,20 +282,26 @@ CC_conninfo_init(ConnInfo *conninfo)
 		conninfo->lower_case_identifier = -1;
 		conninfo->rollback_on_error = -1;
 		conninfo->force_abbrev_connstr = -1;
-		conninfo->fake_mss = -1;
 		conninfo->bde_environment = -1;
+		conninfo->fake_mss = -1;
+		conninfo->cvt_null_date_string = -1;
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 		conninfo->xa_opt = -1;
 		conninfo->autocommit_normal = 0;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 		memcpy(&(conninfo->drivers), &globals, sizeof(globals));
 }
+
+#ifdef	WIN32
+extern	int	platformId;
+#endif /* WIN32 */
 /*
  *		IMPLEMENTATION CONNECTION CLASS
  */
 ConnectionClass *
 CC_Constructor()
 {
+	extern	int	exepgm;
 	ConnectionClass *rv, *retrv = NULL;
 
 	rv = (ConnectionClass *) calloc(sizeof(ConnectionClass), 1);
@@ -307,11 +349,17 @@ CC_Constructor()
 		// rv->DataSourceToDriver = NULL;
 		// rv->DriverToDataSource = NULL;
 		rv->driver_version = ODBCVER;
+#ifdef	WIN32
+		if (VER_PLATFORM_WIN32_WINDOWS == platformId && rv->driver_version > 0x0300)
+			rv->driver_version = 0x0300;
+#endif /* WIN32 */
 		// memset(rv->pg_version, 0, sizeof(rv->pg_version));
 		// rv->pg_version_number = .0;
 		// rv->pg_version_major = 0;
 		// rv->pg_version_minor = 0;
 		// rv->ms_jet = 0;
+		if (1 == exepgm)
+			rv->ms_jet = 1;
 		// rv->unicode = 0;
 		// rv->result_uncommitted = 0;
 		// rv->schema_support = 0;
@@ -335,6 +383,7 @@ CC_Constructor()
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 		// rv->asdum = NULL;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
+		INIT_CONNLOCK(rv);
 		INIT_CONN_CS(rv);
 		retrv = rv;
 	}
@@ -349,7 +398,7 @@ cleanup:
 char
 CC_Destructor(ConnectionClass *self)
 {
-	mylog("enter CC_Destructor, self=%x\n", self);
+	mylog("enter CC_Destructor, self=%p\n", self);
 
 	if (self->status == CONN_EXECUTING)
 		return 0;
@@ -378,6 +427,7 @@ CC_Destructor(ConnectionClass *self)
 	if (self->__error_message)
 		free(self->__error_message);
 	DELETE_CONN_CS(self);
+	DELETE_CONNLOCK(self);
 	free(self);
 
 	mylog("exit CC_Destructor\n");
@@ -395,14 +445,16 @@ CC_cursor_count(ConnectionClass *self)
 				count = 0;
 	QResultClass		*res;
 
-	mylog("CC_cursor_count: self=%x, num_stmts=%d\n", self, self->num_stmts);
+	mylog("CC_cursor_count: self=%p, num_stmts=%d\n", self, self->num_stmts);
 
+	CONNLOCK_ACQUIRE(self);
 	for (i = 0; i < self->num_stmts; i++)
 	{
 		stmt = self->stmts[i];
 		if (stmt && (res = SC_get_Result(stmt)) && QR_get_cursor(res))
 			count++;
 	}
+	CONNLOCK_RELEASE(self);
 
 	mylog("CC_cursor_count: returning %d\n", count);
 
@@ -414,6 +466,7 @@ void
 CC_clear_error(ConnectionClass *self)
 {
 	if (!self)	return;
+	CONNLOCK_ACQUIRE(self);
 	self->__error_number = 0;
 	if (self->__error_message)
 	{
@@ -422,6 +475,7 @@ CC_clear_error(ConnectionClass *self)
 	}
 	self->sqlstate[0] = '\0';
 	self->errormsg_created = FALSE;
+	CONNLOCK_RELEASE(self);
 }
 
 
@@ -494,7 +548,7 @@ CC_cleanup(ConnectionClass *self)
 	if (self->status == CONN_EXECUTING)
 		return FALSE;
 
-	mylog("in CC_Cleanup, self=%x\n", self);
+	mylog("in CC_Cleanup, self=%p\n", self);
 
 	/* Cancel an ongoing transaction */
 	/* We are always in the middle of a transaction, */
@@ -654,8 +708,9 @@ md5_auth_send(ConnectionClass *self, const char *salt)
 	char	*pwd1 = NULL, *pwd2 = NULL;
 	ConnInfo   *ci = &(self->connInfo);
 	SocketClass	*sock = self->sock;
+	size_t		md5len;
 
-inolog("md5 pwd=%s user=%s\n", ci->password, ci->username);
+inolog("md5 pwd=%s user=%s salt=%02x%02x%02x%02x%02x\n", ci->password, ci->username, (UCHAR)salt[0], (UCHAR)salt[1], (UCHAR)salt[2], (UCHAR)salt[3], (UCHAR)salt[4]);
 	if (!(pwd1 = malloc(MD5_PASSWD_LEN + 1)))
 		return 1;
 	if (!EncryptMD5(ci->password, ci->username, strlen(ci->username), pwd1))
@@ -677,12 +732,14 @@ inolog("md5 pwd=%s user=%s\n", ci->password, ci->username);
 	free(pwd1);
 	if (PROTOCOL_74(&(self->connInfo)))
 {
-inolog("putting p\n");
+inolog("putting p and %s\n", pwd2);
 		SOCK_put_char(sock, 'p');
 }
-	SOCK_put_int(sock, 4 + (int) strlen(pwd2) + 1, 4);
-	SOCK_put_n_char(sock, pwd2, (int) strlen(pwd2) + 1);
+	md5len = strlen(pwd2);
+	SOCK_put_int(sock, (Int4) (4 + md5len + 1), 4);
+	SOCK_put_n_char(sock, pwd2, (Int4) (md5len + 1));
 	SOCK_flush_output(sock);
+inolog("sockerr=%d\n", SOCK_get_errcode(sock));
 	free(pwd2);
 	return 0; 
 }
@@ -690,7 +747,7 @@ inolog("putting p\n");
 int
 EatReadyForQuery(ConnectionClass *conn)
 {
-	int	id;
+	int	id = 0;
 
 	if (PROTOCOL_74(&(conn->connInfo)))
 	{
@@ -721,7 +778,7 @@ EatReadyForQuery(ConnectionClass *conn)
 }
 
 int
-handle_error_message(ConnectionClass *self, char *msgbuf, int buflen, char *sqlstate, const char *comment, QResultClass *res)
+handle_error_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *sqlstate, const char *comment, QResultClass *res)
 {
 	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated;
 	SocketClass	*sock = self->sock;
@@ -783,7 +840,7 @@ inolog("new_format=%d\n", new_format);
 	}
 	else
 	{
-		msg_truncated = SOCK_get_string(sock, msgbuf, buflen);
+		msg_truncated = SOCK_get_string(sock, msgbuf, (Int4) buflen);
 
 		/* Remove a newline */
 		if (msgbuf[0] != '\0' && msgbuf[(int)strlen(msgbuf) - 1] == '\n')
@@ -823,7 +880,7 @@ inolog("new_format=%d\n", new_format);
 }
 
 int
-handle_notice_message(ConnectionClass *self, char *msgbuf, int buflen, char *sqlstate, const char *comment, QResultClass *res)
+handle_notice_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *sqlstate, const char *comment, QResultClass *res)
 {
 	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated;
 	SocketClass	*sock = self->sock;
@@ -884,7 +941,7 @@ handle_notice_message(ConnectionClass *self, char *msgbuf, int buflen, char *sql
 	}
 	else
 	{
-		msg_truncated = SOCK_get_string(sock, msgbuf, buflen);
+		msg_truncated = SOCK_get_string(sock, msgbuf, (Int4) buflen);
 
 		/* Remove a newline */
 		if (msgbuf[0] != '\0' && msgbuf[strlen(msgbuf) - 1] == '\n')
@@ -958,11 +1015,9 @@ inolog("parameter value=%s\n", msgbuffer);
 
 static int	protocol3_opts_array(ConnectionClass *self, const char *opts[][2], BOOL libpqopt, int dim_opts)
 {
-	SocketClass	*sock = self->sock;
 	ConnInfo	*ci = &(self->connInfo);
 	const	char	*enc = NULL;
 	int	cnt;
-	BOOL	set_client_encode = FALSE;
 
 	cnt = 0;
 	if (libpqopt && ci->server[0])
@@ -1026,13 +1081,11 @@ static int	protocol3_packet_build(ConnectionClass *self)
 {
 	CSTR	func = "protocol3_packet_build";
 	SocketClass	*sock = self->sock;
-	ConnInfo	*ci = &(self->connInfo);
-	Int4	slen;
+	size_t	slen;
 	char	*packet, *ppacket;
 	ProtocolVersion	pversion;
-	const	char	*opts[20][2], *enc = NULL;
+	const	char	*opts[20][2];
 	int	cnt, i;
-	BOOL	set_client_encode = FALSE;
 
 	cnt = protocol3_opts_array(self, opts, FALSE, sizeof(opts) / sizeof(opts[0]));
 
@@ -1054,7 +1107,7 @@ static int	protocol3_packet_build(ConnectionClass *self)
 
 	sock->pversion = PG_PROTOCOL_LATEST;
 	/* Send length of Authentication Block */
-	SOCK_put_int(sock, slen + 4, 4);
+	SOCK_put_int(sock, (Int4) (slen + 4), 4);
 
 	ppacket = packet;
 	pversion = (ProtocolVersion) htonl(sock->pversion);
@@ -1069,7 +1122,7 @@ static int	protocol3_packet_build(ConnectionClass *self)
 	}
 	*ppacket = '\0';
 
-	SOCK_put_n_char(sock, packet, slen);
+	SOCK_put_n_char(sock, packet, (Int4) slen);
 	SOCK_flush_output(sock);
 	free(packet);
 
@@ -1080,8 +1133,7 @@ CSTR	l_login_timeout = "connect_timeout";
 static char	*protocol3_opts_build(ConnectionClass *self)
 {
 	CSTR	func = "protocol3_opts_build";
-	ConnInfo	*ci = &(self->connInfo);
-	Int4	slen;
+	size_t	slen;
 	char	*conninfo, *ppacket;
 	const	char	*opts[20][2];
 	int	cnt, i;
@@ -1100,7 +1152,7 @@ static char	*protocol3_opts_build(ConnectionClass *self)
 		char	tmout[16];
 
 		slen += (strlen(l_login_timeout) + 2 + 2);
-		snprintf(tmout, sizeof(tmout), "%d", self->login_timeout);
+		snprintf(tmout, sizeof(tmout), FORMAT_UINTEGER, self->login_timeout);
 		slen += strlen(tmout);
 	}
 	slen++;
@@ -1137,7 +1189,7 @@ static char	*protocol3_opts_build(ConnectionClass *self)
 	{
 		sprintf(ppacket, " %s=", l_login_timeout);
 		ppacket += (strlen(l_login_timeout) + 2);
-		sprintf(ppacket, "%d", self->login_timeout);
+		sprintf(ppacket, FORMAT_UINTEGER, self->login_timeout);
 		ppacket = strchr(ppacket, '\0');
 	}
 	*ppacket = '\0';
@@ -1189,6 +1241,8 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 		return 0;
 	}
 
+	mylog("%s: DSN = '%s', server = '%s', port = '%s', database = '%s', username = '%s', password='%s'\n", func, ci->dsn, ci->server, ci->port, ci->database, ci->username, ci->password ? "xxxxx" : "");
+
 	if (ci->port[0] == '\0' ||
 #ifdef	WIN32
 	    ci->server[0] == '\0' ||
@@ -1199,8 +1253,6 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 		return 0;
 	}
 
-	mylog("%s: DSN = '%s', server = '%s', port = '%s', database = '%s', username = '%s', password='%s'\n", func, ci->dsn, ci->server, ci->port, ci->database, ci->username, ci->password ? "xxxxx" : "");
-
 	return 1;
 }
 
@@ -1209,7 +1261,6 @@ static int LIBPQ_connect(ConnectionClass *self);
 static char
 LIBPQ_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 {
-	ConnInfo   *ci = &(self->connInfo);
 	int		ret;
 	CSTR		func = "LIBPQ_CC_connect";
 
@@ -1237,7 +1288,7 @@ original_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 	SocketClass *sock;
 	ConnInfo   *ci = &(self->connInfo);
 	int			areq = -1;
-	int			beresp;
+	int			beresp, sockerr;
 	char		msgbuffer[ERROR_MSG_LENGTH];
 	char		salt[5], notice[512];
 	CSTR		func = "original_CC_connect";
@@ -1247,9 +1298,10 @@ original_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 	mylog("%s: entering...\n", func);
 
 	if (password_req != AUTH_REQ_OK)
-
+	{
 		sock = self->sock;		/* already connected, just authenticate */
-
+		CC_clear_error(self);	
+	}
 	else
 	{
 		if (0 == CC_initial_log(self, func))
@@ -1325,11 +1377,9 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 			SOCK_flush_output(sock);
 		}
 
-		mylog("sent the authentication block.\n");
-
-		if (sock->errornumber != 0)
+		if (SOCK_get_errcode(sock) != 0)
 		{
-			CC_set_error(self, CONN_INVALID_AUTHENTICATION, "Sending the authentication packet failed", func);
+			CC_set_error(self, CONN_INVALID_AUTHENTICATION, "Failed to send the authentication packet", func);
 			return 0;
 		}
 		mylog("sent the authentication block successfully.\n");
@@ -1352,20 +1402,28 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 		do
 		{
 			if (password_req != AUTH_REQ_OK)
+			{
 				beresp = 'R';
+				startPacketReceived = TRUE;
+			}
 			else
 			{
 				beresp = SOCK_get_id(sock);
 				mylog("auth got '%c'\n", beresp);
+				if (0 != SOCK_get_errcode(sock))
+					goto sockerr_proc;
 				if (PROTOCOL_74(ci))
 				{
 					if (beresp != 'E' || startPacketReceived)
 					{
 						leng = SOCK_get_response_length(sock);
 inolog("leng=%d\n", leng);
+						if (0 != SOCK_get_errcode(sock))
+							goto sockerr_proc;
 					}
 					else
-						strcpy(ci->protocol, PG74REJECTED);
+						strncpy(ci->protocol, PG74REJECTED, sizeof(ci->protocol));
+/* retry = TRUE; */
 				}
 				startPacketReceived = TRUE;
 			}
@@ -1383,18 +1441,7 @@ inolog("Ekita\n");
 						 strnicmp(msgbuffer + 8, "unsupported frontend protocol", 29) == 0)
 						retry = TRUE;
 					if (retry)
-					{			/* retry older version */
-						if (PROTOCOL_63(ci))
-							strcpy(ci->protocol, PG62);
-						else if (PROTOCOL_64(ci))
-							strcpy(ci->protocol, PG63);
-						else 
-							strcpy(ci->protocol, PG64);
-						SOCK_Destructor(sock);
-						self->sock = (SocketClass *) 0;
-						CC_initialize_pg_version(self);
-						goto another_version_retry;
-					}
+						break;
 
 					return 0;
 				case 'R':
@@ -1406,17 +1453,19 @@ inolog("Ekita\n");
 						if (salt_para)
 							memcpy(salt, salt_para, sizeof(salt));
 						password_req = AUTH_REQ_OK;
+						mylog("salt=%02x%02x%02x%02x%02x\n", (UCHAR)salt[0], (UCHAR)salt[1], (UCHAR)salt[2], (UCHAR)salt[3], (UCHAR)salt[4]);
 					}
 					else
 					{
 
 						areq = SOCK_get_int(sock, 4);
+						memset(salt, 0, sizeof(salt));
 						if (areq == AUTH_REQ_MD5)
 							SOCK_get_n_char(sock, salt, 4);
 						else if (areq == AUTH_REQ_CRYPT)
 							SOCK_get_n_char(sock, salt, 2);
 
-						mylog("areq = %d\n", areq);
+						mylog("areq = %d salt=%02x%02x%02x%02x%02x\n", areq, (UCHAR)salt[0], (UCHAR)salt[1], (UCHAR)salt[2], (UCHAR)salt[3], (UCHAR)salt[4]);
 					}
 					switch (areq)
 					{
@@ -1442,11 +1491,13 @@ inolog("Ekita\n");
 
 							mylog("past need password\n");
 
-							SOCK_put_int(sock, 4 + strlen(ci->password) + 1, 4);
-							SOCK_put_n_char(sock, ci->password, strlen(ci->password) + 1);
-							SOCK_flush_output(sock);
+							if (PROTOCOL_74(&(self->connInfo)))
+								SOCK_put_char(sock, 'p');
+							SOCK_put_int(sock, (Int4) (4 + strlen(ci->password) + 1), 4);
+							SOCK_put_n_char(sock, ci->password, (Int4) strlen(ci->password) + 1);
+							sockerr = SOCK_flush_output(sock);
 
-							mylog("past flush\n");
+							mylog("past flush %dbytes\n", sockerr);
 							break;
 
 						case AUTH_REQ_CRYPT:
@@ -1497,6 +1548,19 @@ inolog("Ekita\n");
 					CC_set_error(self, CONN_INVALID_AUTHENTICATION, notice, func);
 					return 0;
 			}
+			if (retry)
+			{	/* retry older version */
+				if (PROTOCOL_63(ci))
+					strncpy(ci->protocol, PG62, sizeof(ci->protocol));
+				else if (PROTOCOL_64(ci))
+					strncpy(ci->protocol, PG63, sizeof(ci->protocol));
+				else 
+					strncpy(ci->protocol, PG64, sizeof(ci->protocol));
+				SOCK_Destructor(sock);
+				self->sock = (SocketClass *) 0;
+				CC_initialize_pg_version(self);
+				goto another_version_retry;
+			}
 
 			/*
 			 * There were no ReadyForQuery responce before 6.4.
@@ -1506,6 +1570,18 @@ inolog("Ekita\n");
 		} while (!ReadyForQuery);
 	}
 
+sockerr_proc:
+	if (0 != (sockerr = SOCK_get_errcode(sock)))
+	{
+		if (0 == CC_get_errornumber(self))
+		{
+			if (SOCKET_CLOSED == sockerr)
+				CC_set_error(self, CONN_INVALID_AUTHENTICATION, "Communication closed during authentication", func);
+			else
+				CC_set_error(self, CONN_INVALID_AUTHENTICATION, "Communication error during authentication", func);
+		}
+		return 0;
+	}
 
 	CC_clear_error(self);		/* clear any password error */
 
@@ -1559,7 +1635,6 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 	char	   ret;
 	// char	   *encoding;
 	// BOOL	startPacketReceived = FALSE;
-	BOOL	usessl = TRUE;
 
 	mylog("%s: entering...\n", func);
 
@@ -1674,31 +1749,38 @@ mylog("conn->unicode=%d\n", self->unicode);
 char
 CC_add_statement(ConnectionClass *self, StatementClass *stmt)
 {
-	int			i;
+	int	i;
+	char	ret = TRUE;
 
-	mylog("CC_add_statement: self=%x, stmt=%x\n", self, stmt);
+	mylog("CC_add_statement: self=%p, stmt=%p\n", self, stmt);
 
+	CONNLOCK_ACQUIRE(self);
 	for (i = 0; i < self->num_stmts; i++)
 	{
 		if (!self->stmts[i])
 		{
 			stmt->hdbc = self;
 			self->stmts[i] = stmt;
-			return TRUE;
+			break;
 		}
 	}
 
-	/* no more room -- allocate more memory */
-	self->stmts = (StatementClass **) realloc(self->stmts, sizeof(StatementClass *) * (STMT_INCREMENT + self->num_stmts));
-	if (!self->stmts)
-		return FALSE;
+	if (i >= self->num_stmts) /* no more room -- allocate more memory */
+	{
+		self->stmts = (StatementClass **) realloc(self->stmts, sizeof(StatementClass *) * (STMT_INCREMENT + self->num_stmts));
+		if (!self->stmts)
+			ret = FALSE;
+		else
+		{
+			memset(&self->stmts[self->num_stmts], 0, sizeof(StatementClass *) * STMT_INCREMENT);
 
-	memset(&self->stmts[self->num_stmts], 0, sizeof(StatementClass *) * STMT_INCREMENT);
+			stmt->hdbc = self;
+			self->stmts[self->num_stmts] = stmt;
 
-	stmt->hdbc = self;
-	self->stmts[self->num_stmts] = stmt;
-
-	self->num_stmts += STMT_INCREMENT;
+			self->num_stmts += STMT_INCREMENT;
+		}
+	}
+	CONNLOCK_RELEASE(self);
 
 	return TRUE;
 }
@@ -1708,7 +1790,7 @@ CC_set_error_statements(ConnectionClass *self)
 {
 	int	i;
 
-	mylog("CC_error_statements: self=%x\n", self);
+	mylog("CC_error_statements: self=%p\n", self);
 
 	for (i = 0; i < self->num_stmts; i++)
 	{
@@ -1721,18 +1803,22 @@ CC_set_error_statements(ConnectionClass *self)
 char
 CC_remove_statement(ConnectionClass *self, StatementClass *stmt)
 {
-	int			i;
+	int	i;
+	char	ret = FALSE;
 
+	CONNLOCK_ACQUIRE(self);
 	for (i = 0; i < self->num_stmts; i++)
 	{
 		if (self->stmts[i] == stmt && stmt->status != STMT_EXECUTING)
 		{
 			self->stmts[i] = NULL;
-			return TRUE;
+			ret = TRUE;
+			break;
 		}
 	}
+	CONNLOCK_RELEASE(self);
 
-	return FALSE;
+	return ret;
 }
 
 int	CC_get_max_idlen(ConnectionClass *self)
@@ -1756,12 +1842,13 @@ mylog("max_identifier_length=%d\n", len);
  *	Create a more informative error message by concatenating the connection
  *	error message with its socket error message.
  */
-char *
+static char *
 CC_create_errormsg(ConnectionClass *self)
 {
 	SocketClass *sock = self->sock;
-	int			pos;
-	char	 msg[4096];
+	size_t	pos;
+	char	msg[4096];
+	const char *sockerrmsg;
 
 	mylog("enter CC_create_errormsg\n");
 
@@ -1772,10 +1859,10 @@ CC_create_errormsg(ConnectionClass *self)
 
 	mylog("msg = '%s'\n", msg);
 
-	if (sock && sock->errormsg && sock->errormsg[0] != '\0')
+	if (sock && NULL != (sockerrmsg = SOCK_get_errmsg(sock)) && '\0' != sockerrmsg[0])
 	{
 		pos = strlen(msg);
-		snprintf(&msg[pos], sizeof(msg) - pos, ";\n%s", sock->errormsg);
+		snprintf(&msg[pos], sizeof(msg) - pos, ";\n%s", sockerrmsg);
 	}
 
 	mylog("exit CC_create_errormsg\n");
@@ -1786,6 +1873,7 @@ CC_create_errormsg(ConnectionClass *self)
 void
 CC_set_error(ConnectionClass *self, int number, const char *message, const char *func)
 {
+	CONNLOCK_ACQUIRE(self);
 	if (self->__error_message)
 		free(self->__error_message);
 	self->__error_number = number;
@@ -1794,15 +1882,18 @@ CC_set_error(ConnectionClass *self, int number, const char *message, const char
 		CC_set_error_statements(self);
 	if (func && number != 0)
 		CC_log_error(func, "", self); 
+	CONNLOCK_RELEASE(self);
 }
 
 
 void
 CC_set_errormsg(ConnectionClass *self, const char *message)
 {
+	CONNLOCK_ACQUIRE(self);
 	if (self->__error_message)
 		free(self->__error_message);
 	self->__error_message = message ? strdup(message) : NULL;
+	CONNLOCK_RELEASE(self);
 }
 
 
@@ -1814,6 +1905,7 @@ CC_get_error(ConnectionClass *self, int *number, char **message)
 
 	mylog("enter CC_get_error\n");
 
+	CONNLOCK_ACQUIRE(self);
 	/* Create a very informative errormsg if it hasn't been done yet. */
 	if (!self->errormsg_created)
 	{
@@ -1832,6 +1924,7 @@ CC_get_error(ConnectionClass *self, int *number, char **message)
 	rv = (CC_get_errornumber(self) != 0);
 
 	self->__error_number = 0;		/* clear the error */
+	CONNLOCK_RELEASE(self);
 
 	mylog("exit CC_get_error\n");
 
@@ -1847,6 +1940,7 @@ static void CC_clear_cursors(ConnectionClass *self, BOOL on_abort)
 
 	if (!self->ncursors)
 		return;
+	CONNLOCK_ACQUIRE(self);
 	for (i = 0; i < self->num_stmts; i++)
 	{
 		stmt = self->stmts[i];
@@ -1868,37 +1962,47 @@ static void CC_clear_cursors(ConnectionClass *self, BOOL on_abort)
 				char	cmd[64];
 
 				snprintf(cmd, sizeof(cmd), "MOVE 0 in \"%s\"", QR_get_cursor(res));
+				CONNLOCK_RELEASE(self);
 				wres = CC_send_query(self, cmd, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
 				if (QR_command_maybe_successful(wres))
 					QR_set_permanent(res);
 				else
 					QR_set_cursor(res, NULL);
 				QR_Destructor(wres);
+				CONNLOCK_ACQUIRE(self);
 			}
 		}
 	}
+	CONNLOCK_RELEASE(self);
 }
 	
 void	CC_on_commit(ConnectionClass *conn)
 {
+	CONNLOCK_ACQUIRE(conn);
 	if (CC_is_in_trans(conn))
 	{
 		CC_set_no_trans(conn);
 		CC_set_no_manual_trans(conn);
 	}
 	CC_clear_cursors(conn, FALSE);
+	CONNLOCK_RELEASE(conn);
 	CC_discard_marked_objects(conn);
+	CONNLOCK_ACQUIRE(conn);
 	if (conn->result_uncommitted)
 	{
+		CONNLOCK_RELEASE(conn);
 		ProcessRollback(conn, FALSE, FALSE);
+		CONNLOCK_ACQUIRE(conn);
 		conn->result_uncommitted = 0;
 	}
+	CONNLOCK_RELEASE(conn);
 }
 void	CC_on_abort(ConnectionClass *conn, UDWORD opt)
 {
 	BOOL	set_no_trans = FALSE;
 
 mylog("CC_on_abort in\n");
+	CONNLOCK_ACQUIRE(conn);
 	if (0 != (opt & CONN_DEAD)) /* CONN_DEAD implies NO_TRANS also */
 		opt |= NO_TRANS;
 	if (CC_is_in_trans(conn))
@@ -1916,24 +2020,35 @@ mylog("CC_on_abort in\n");
 		conn->status = CONN_DOWN;
 		if (conn->sock)
 		{
+			CONNLOCK_RELEASE(conn);
 			SOCK_Destructor(conn->sock);
+			CONNLOCK_ACQUIRE(conn);
 			conn->sock = NULL;
 		}
 	}
 	else if (set_no_trans)
+	{
+		CONNLOCK_RELEASE(conn);
 		CC_discard_marked_objects(conn);
+		CONNLOCK_ACQUIRE(conn);
+	}
 	if (conn->result_uncommitted)
 	{
+		CONNLOCK_RELEASE(conn);
 		ProcessRollback(conn, TRUE, FALSE);
+		CONNLOCK_ACQUIRE(conn);
 		conn->result_uncommitted = 0;
 	}
+	CONNLOCK_RELEASE(conn);
 }
 
 void	CC_on_abort_partial(ConnectionClass *conn)
 {
 mylog("CC_on_abort_partial in\n");
 	ProcessRollback(conn, TRUE, TRUE);
+	CONNLOCK_ACQUIRE(conn);
 	CC_discard_marked_objects(conn);
+	CONNLOCK_RELEASE(conn);
 }
 
 /*
@@ -1983,8 +2098,8 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag, St
 	/* QR_set_command() dups this string so doesn't need static */
 	char		cmdbuffer[ERROR_MSG_LENGTH + 1];
 
-	mylog("send_query(): conn=%x, query='%s'\n", self, query);
-	qlog("conn=%x, query='%s'\n", self, query);
+	mylog("send_query(): conn=%p, query='%s'\n", self, query);
+	qlog("conn=%p, query='%s'\n", self, query);
 
 	if (!self->sock)
 	{
@@ -2018,7 +2133,7 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag, St
 	if (rollback_on_error)
 		rollback_on_error = consider_rollback;
 	query_rollback = (rollback_on_error && PG_VERSION_GE(self, 8.0));
-	if (!query_rollback && consider_rollback )
+	if (!query_rollback && consider_rollback)
 	{
 		if (stmt)
 		{
@@ -2041,14 +2156,16 @@ CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag, St
 		CC_on_abort(self, CONN_DEAD);
 		goto cleanup;
 	}
+	if (stmt)
+		SC_forget_unnamed(stmt);
 
 	if (PROTOCOL_74(ci))
 	{
-		leng = strlen(query);
+		leng = (UInt4) strlen(query);
 		if (issue_begin)
-			leng += strlen("BEGIN;");
+			leng += (UInt4) strlen("BEGIN;");
 		if (query_rollback)
-			leng += (10 + strlen(per_query_svp) + 1);
+			leng += (UInt4) (10 + strlen(per_query_svp) + 1);
 		leng++;
 		SOCK_put_int(sock, leng + 4, 4);
 inolog("leng=%d\n", leng);
@@ -2063,11 +2180,11 @@ inolog("leng=%d\n", leng);
 		char cmd[64];
 
 		snprintf(cmd, sizeof(cmd), "SAVEPOINT %s;", per_query_svp);
-		SOCK_put_n_char(sock, cmd, strlen(cmd));
+		SOCK_put_n_char(sock, cmd, (Int4) strlen(cmd));
 		discard_next_savepoint = TRUE;
 	}
 	SOCK_put_string(sock, query);
-	SOCK_flush_output(sock);
+	leng = SOCK_flush_output(sock);
 
 	if (SOCK_get_errcode(sock) != 0)
 	{
@@ -2076,7 +2193,7 @@ inolog("leng=%d\n", leng);
 		goto cleanup;
 	}
 
-	mylog("send_query: done sending query\n");
+	mylog("send_query: done sending query %dbytes flushed\n", leng);
  
 	empty_reqs = 0;
 	for (wq = query; isspace((UCHAR) *wq); wq++)
@@ -2195,7 +2312,7 @@ inolog("Discarded the first SAVEPOINT\n");
 						QR_set_rstatus(res, PORES_COMMAND_OK);
 					QR_set_command(res, cmdbuffer);
 					query_completed = TRUE;
-					mylog("send_query: returning res = %x\n", res);
+					mylog("send_query: returning res = %p\n", res);
 					if (!before_64)
 						break;
 
@@ -2286,7 +2403,7 @@ inolog("Discarded the first SAVEPOINT\n");
 						if (stmt)
 							res->num_key_fields = stmt->num_key_fields;
 					}
-					mylog("send_query: 'T' no result_in: res = %x\n", res->next);
+					mylog("send_query: 'T' no result_in: res = %p\n", res->next);
 					res = res->next;
 
 					if (qi)
@@ -2359,7 +2476,7 @@ inolog("Discarded the first SAVEPOINT\n");
 				QR_set_no_fetching_tuples(res);
 				break;
 			default:
-				/* skip the unexpecte response if possible */
+				/* skip the unexpected response if possible */
 				if (response_length >= 0)
 					break;
 				CC_set_error(self, CONNECTION_BACKEND_CRAZY, "Unexpected protocol character from backend (send_query)", func);
@@ -2371,6 +2488,8 @@ inolog("Discarded the first SAVEPOINT\n");
 				break;
 		}
 
+		if (SOCK_get_errcode(sock) != 0)
+			break;
 		if (CONN_DOWN == self->status)
 			break;
 		/*
@@ -2384,6 +2503,14 @@ inolog("Discarded the first SAVEPOINT\n");
 	}
 
 cleanup:
+	if (SOCK_get_errcode(sock) != 0)
+	{
+		if (0 == CC_get_errornumber(self))
+			CC_set_error(self, CONNECTION_COMMUNICATION_ERROR, "Communication error while sending query", func);
+		CC_on_abort(self, CONN_DEAD);
+		ReadyToReturn = TRUE;
+		retres = NULL;
+	}
 	if (rollback_on_error && CC_is_in_trans(self) && !discard_next_savepoint)
 	{
 		char	cmd[64];
@@ -2480,7 +2607,7 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 	ConnInfo		*ci;
 	int			func_cs_count = 0; 
 
-	mylog("send_function(): conn=%x, fnid=%d, result_is_int=%d, nargs=%d\n", self, fnid, result_is_int, nargs);
+	mylog("send_function(): conn=%p, fnid=%d, result_is_int=%d, nargs=%d\n", self, fnid, result_is_int, nargs);
 
 	if (!self->sock)
 	{
@@ -2520,7 +2647,7 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 		SOCK_put_int(sock, leng, 4);
 	}
 	else
-	SOCK_put_string(sock, "F ");
+		SOCK_put_string(sock, "F ");
 	if (SOCK_get_errcode(sock) != 0)
 	{
 		CC_set_error(self, CONNECTION_COULD_NOT_SEND, "Could not send function to backend", func);
@@ -2537,13 +2664,13 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 		SOCK_put_int(sock, nargs, 2);
 	}
 	else
-	SOCK_put_int(sock, nargs, 4);
+		SOCK_put_int(sock, nargs, 4);
 
 	mylog("send_function: done sending function\n");
 
 	for (i = 0; i < nargs; ++i)
 	{
-		mylog("  arg[%d]: len = %d, isint = %d, integer = %d, ptr = %x\n", i, args[i].len, args[i].isint, args[i].u.integer, args[i].u.ptr);
+		mylog("  arg[%d]: len = %d, isint = %d, integer = %d, ptr = %p\n", i, args[i].len, args[i].isint, args[i].u.integer, args[i].u.ptr);
 
 		SOCK_put_int(sock, args[i].len, 4);
 		if (args[i].isint)
@@ -2651,14 +2778,10 @@ CC_setenv(ConnectionClass *self)
 {
 	ConnInfo   *ci = &(self->connInfo);
 
-/* QResultClass *res; */
 	HSTMT		hstmt;
 	StatementClass *stmt;
 	RETCODE		result;
 	char		status = TRUE;
-#ifdef	HAVE_STRTOK_R
-	char	*last;
-#endif /* HAVE_STRTOK_R */
 	CSTR func = "CC_setenv";
 
 
@@ -2819,7 +2942,7 @@ CC_send_settings(ConnectionClass *self)
  *	If a real Large Object oid type is made part of Postgres, this function
  *	will go away and the define 'PG_TYPE_LO' will be updated.
  */
-void
+static void
 CC_lookup_lo(ConnectionClass *self)
 {
 	QResultClass	*res;
@@ -2892,7 +3015,7 @@ CC_initialize_pg_version(ConnectionClass *self)
  *	This is used to return the correct info in SQLGetInfo
  *	DJP - 25-1-2001
  */
-void
+static void
 CC_lookup_pg_version(ConnectionClass *self)
 {
 	HSTMT		hstmt;
@@ -2972,15 +3095,15 @@ CC_log_error(const char *func, const char *desc, const ConnectionClass *self)
 		qlog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
 		mylog("CONN ERROR: func=%s, desc='%s', errnum=%d, errmsg='%s'\n", func, desc, self->__error_number, nullcheck(self->__error_message));
 		qlog("            ------------------------------------------------------------\n");
-		qlog("            henv=%x, conn=%x, status=%u, num_stmts=%d\n", self->henv, self, self->status, self->num_stmts);
-		qlog("            sock=%x, stmts=%x, lobj_type=%d\n", self->sock, self->stmts, self->lobj_type);
+		qlog("            henv=%p, conn=%p, status=%u, num_stmts=%d\n", self->henv, self, self->status, self->num_stmts);
+		qlog("            sock=%p, stmts=%p, lobj_type=%d\n", self->sock, self->stmts, self->lobj_type);
 
 		qlog("            ---------------- Socket Info -------------------------------\n");
 		if (self->sock)
 		{
 			SocketClass *sock = self->sock;
 
-			qlog("            socket=%d, reverse=%d, errornumber=%d, errormsg='%s'\n", sock->socket, sock->reverse, sock->errornumber, nullcheck(sock->errormsg));
+			qlog("            socket=%d, reverse=%d, errornumber=%d, errormsg='%s'\n", sock->socket, sock->reverse, sock->errornumber, nullcheck(SOCK_get_errmsg(sock)));
 			qlog("            buffer_in=%u, buffer_out=%u\n", sock->buffer_in, sock->buffer_out);
 			qlog("            buffer_filled_in=%d, buffer_filled_out=%d, buffer_read_in=%d\n", sock->buffer_filled_in, sock->buffer_filled_out, sock->buffer_read_in);
 		}
@@ -3155,7 +3278,7 @@ LIBPQ_connect(ConnectionClass *self)
 
 	mylog("connecting to the database  using %s as the server\n",self->connInfo.server);
 	sock = self->sock;
-inolog("sock=%x\n", sock);
+inolog("sock=%p\n", sock);
 	if (!sock)
 	{
 		sock = SOCK_Constructor(self);
@@ -3218,18 +3341,19 @@ cleanup1:
 inolog("socket=%d\n", socket);
 	sock->socket = socket;
 	sock->ssl = PQgetssl(pqconn);
+if (TRUE)
 	{
 		int	pversion;
 		ConnInfo	*ci = &self->connInfo;
 
 		sock->pversion = PG_PROTOCOL_74;
-		strcpy(ci->protocol, PG74);
+		strncpy(ci->protocol, PG74, sizeof(ci->protocol));
 		pversion = PQprotocolVersion(pqconn);
 		switch (pversion)
 		{
 			case 2:
 				sock->pversion = PG_PROTOCOL_64;
-				strcpy(ci->protocol, PG64);
+				strncpy(ci->protocol, PG64, sizeof(ci->protocol));
 				break;
 		}
 	}
