@@ -38,20 +38,20 @@ typedef struct
 	int			fr;
 } SIMPLE_TIME;
 
-int		copy_and_convert_field_bindinfo(StatementClass *stmt, Int4 field_type, void *value, int col);
-int copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2 fCType,
+int		copy_and_convert_field_bindinfo(StatementClass *stmt, OID field_type, void *value, int col);
+int copy_and_convert_field(StatementClass *stmt, OID field_type, void *value, SQLSMALLINT fCType,
 			PTR rgbValue, SQLLEN cbValueMax, SQLLEN *pcbValue);
 
 int		copy_statement_with_parameters(StatementClass *stmt, BOOL);
 BOOL		convert_money(const char *s, char *sout, size_t soutmax);
 char		parse_datetime(const char *buf, SIMPLE_TIME *st);
-int			convert_linefeeds(const char *s, char *dst, size_t max, BOOL convlf, BOOL *changed);
-int		convert_special_chars(const char *si, char *dst, int used, UInt4 flags,int ccsc, int escape_ch);
+size_t		convert_linefeeds(const char *s, char *dst, size_t max, BOOL convlf, BOOL *changed);
+size_t		convert_special_chars(const char *si, char *dst, SQLLEN used, UInt4 flags,int ccsc, int escape_ch);
 
-int			convert_pgbinary_to_char(const char *value, char *rgbValue, int cbValueMax);
-int			convert_from_pgbinary(const UCHAR *value, UCHAR *rgbValue, int cbValueMax);
-int		pg_hex2bin(const UCHAR *in, UCHAR *out, int len);
-int convert_lo(StatementClass *stmt, const void *value, Int2 fCType,
+int		convert_pgbinary_to_char(const char *value, char *rgbValue, ssize_t cbValueMax);
+size_t		convert_from_pgbinary(const UCHAR *value, UCHAR *rgbValue, SQLLEN cbValueMax);
+SQLLEN		pg_hex2bin(const UCHAR *in, UCHAR *out, SQLLEN len);
+int convert_lo(StatementClass *stmt, const void *value, SQLSMALLINT fCType,
 	 PTR rgbValue, SQLLEN cbValueMax, SQLLEN *pcbValue);
 
 #ifdef	__cplusplus
