@@ -17,6 +17,7 @@
  */
 /* Multibyte support	Eiji Tokuya 2001-03-15 */
 
+#include <ctype.h>
 #include "dlg_specific.h"
 
 #include "convert.h"
@@ -39,6 +40,8 @@ UInt4	getExtraOptions(const ConnInfo *ci)
 		flag |= BIT_FAKE_MSS;
 	if (ci->bde_environment)
 		flag |= BIT_BDE_ENVIRONMENT;
+	if (ci->cvt_null_date_string)
+		flag |= BIT_CVT_NULL_DATE;
 		
 	return flag;
 }
@@ -52,8 +55,10 @@ static UInt4	replaceExtraOptions(ConnInfo *ci, UInt4 flag, BOOL overwrite)
 		ci->force_abbrev_connstr = (0 != (flag & BIT_FORCEABBREVCONNSTR));
 	if (overwrite || ci->fake_mss < 0)
 		ci->fake_mss = (0 != (flag & BIT_FAKE_MSS));
-	if (overwrite || ci->bde_environment)
+	if (overwrite || ci->bde_environment < 0)
 		ci->bde_environment = (0 != (flag & BIT_BDE_ENVIRONMENT));
+	if (overwrite || ci->cvt_null_date_string < 0)
+		ci->cvt_null_date_string = (0 != (flag & BIT_CVT_NULL_DATE));
 		
 	return getExtraOptions(ci);
 }
@@ -97,12 +102,14 @@ UInt4	add_removeExtraOptions(ConnInfo *ci, UInt4 aflag, UInt4 dflag)
 		ci->fake_mss = TRUE;
 	if (0 != (aflag & BIT_BDE_ENVIRONMENT))
 		ci->bde_environment = TRUE;
+	if (0 != (aflag & BIT_CVT_NULL_DATE))
+		ci->cvt_null_date_string = TRUE;
 	if (0 != (dflag & BIT_FORCEABBREVCONNSTR))
 		ci->force_abbrev_connstr = FALSE;
 	if (0 != (dflag & BIT_FAKE_MSS))
 		ci->fake_mss =FALSE;
-	if (0 != (dflag & BIT_BDE_ENVIRONMENT))
-		ci->bde_environment = FALSE;
+	if (0 != (dflag & BIT_CVT_NULL_DATE))
+		ci->cvt_null_date_string = FALSE;
 
 	return getExtraOptions(ci);
 }
@@ -112,7 +119,7 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 {
 	char		got_dsn = (ci->dsn[0] != '\0');
 	char		encoded_conn_settings[LARGE_REGISTRY_LEN];
-	Int4		hlen, nlen, olen;
+	ssize_t		hlen, nlen, olen;
 	/*BOOL		abbrev = (len <= 400);*/
 	BOOL		abbrev = (len < 1024) || 0 < ci->force_abbrev_connstr;
 	UInt4		flag;
@@ -221,7 +228,7 @@ inolog("hlen=%d", hlen);
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 				);
 	}
-	/* Abbrebiation is needed ? */
+	/* Abbreviation is needed ? */
 	if (abbrev || olen >= nlen || olen < 0)
 	{
 		flag = 0;
@@ -300,7 +307,7 @@ inolog("hlen=%d", hlen);
 				ABBR_MAXLONGVARCHARSIZE "=%d;"
 				INI_INT8AS "=%d;"
 				ABBR_EXTRASYSTABLEPREFIXES "=%s;"
-				INI_ABBREVIATE "=%02x%lx",
+				INI_ABBREVIATE "=%02x%x",
 				encoded_conn_settings,
 				ci->drivers.fetch_max,
 				ci->drivers.socket_buffersize,
@@ -335,7 +342,7 @@ inolog("hlen=%d", hlen);
 			hlen = strlen(connect_string);
 			nlen = MAX_CONNECT_STRING - hlen;
 			olen = snprintf(&connect_string[hlen], nlen, ";"
-				INI_EXTRAOPTIONS "=%lx;",
+				INI_EXTRAOPTIONS "=%x;",
 				flag);
 		}
 	}
@@ -352,7 +359,7 @@ unfoldCXAttribute(ConnInfo *ci, const char *value)
 	if (strlen(value) < 2)
 	{
 		count = 3;
-		sscanf(value, "%lx", &flag);
+		sscanf(value, "%x", &flag);
 	}
 	else
 	{
@@ -360,7 +367,7 @@ unfoldCXAttribute(ConnInfo *ci, const char *value)
 		memcpy(cnt, value, 2);
 		cnt[2] = '\0';
 		sscanf(cnt, "%x", &count);
-		sscanf(value + 2, "%lx", &flag);
+		sscanf(value + 2, "%x", &flag);
 	}
 	ci->disallow_premature = (char)((flag & BIT_DISALLOWPREMATURE) != 0);
 	ci->allow_keyset = (char)((flag & BIT_UPDATABLECURSORS) != 0);
@@ -420,7 +427,7 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 	else if (stricmp(attribute, INI_SERVER) == 0 || stricmp(attribute, SPEC_SERVER) == 0)
 		strcpy(ci->server, value);
 
-	else if (stricmp(attribute, INI_USER) == 0 || stricmp(attribute, "uid") == 0)
+	else if (stricmp(attribute, INI_USER) == 0 || stricmp(attribute, INI_UID) == 0)
 		strcpy(ci->username, value);
 
 	else if (stricmp(attribute, INI_PASSWORD) == 0 || stricmp(attribute, "pwd") == 0)
@@ -439,11 +446,11 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ptr = strchr(value, '-');
 		if (ptr)
 		{
+			*ptr = '\0';
 			if ('-' != *value)
 			{
 				strcpy(ci->protocol, value);
 			}
-			*ptr = '\0';
 			ci->rollback_on_error = atoi(ptr + 1);
 			mylog("rollback_on_error=%d\n", ci->rollback_on_error);
 		}
@@ -527,7 +534,7 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		{
 			setExtraOptions(ci, value, hex_format);
 		}
-		mylog("force_abbrev=%d bde=%d\n", ci->force_abbrev_connstr, ci->bde_environment);
+		mylog("force_abbrev=%d bde=%d cvt_null_date=%x\n", ci->force_abbrev_connstr, ci->bde_environment, ci->cvt_null_date_string);
 	}
 
 	mylog("copyAttributes: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',protocol='%s',conn_settings='%s',disallow_premature=%d)\n", ci->dsn, ci->server, ci->database, ci->username, ci->password ? "xxxxx" : "", ci->port, ci->onlyread, ci->protocol, ci->conn_settings, ci->disallow_premature);
@@ -646,6 +653,8 @@ getDSNdefaults(ConnInfo *ci)
 		ci->fake_mss = 0;
 	if (ci->bde_environment < 0)
 		ci->bde_environment = 0;
+	if (ci->cvt_null_date_string < 0)
+		ci->cvt_null_date_string = 0;
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	if (ci->xa_opt < 0)
 		ci->xa_opt = DEFAULT_XAOPT;
@@ -729,7 +738,7 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 	{
 		char	*ptr;
 		SQLGetPrivateProfileString(DSN, INI_PROTOCOL, "", ci->protocol, sizeof(ci->protocol), ODBC_INI);
-		if (ptr = strchr(ci->protocol, '-'))
+		if (ptr = strchr(ci->protocol, '-'), NULL != ptr)
 		{
 			*ptr = '\0';
 			if (overwrite || ci->rollback_on_error < 0)
@@ -829,7 +838,7 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 
 		sscanf(temp, "%x", &val);
 		replaceExtraOptions(ci, val, overwrite);
-		mylog("force_abbrev=%d bde=%d\n", ci->force_abbrev_connstr, ci->bde_environment);
+		mylog("force_abbrev=%d bde=%d cvt_null_date=%d\n", ci->force_abbrev_connstr, ci->bde_environment, ci->cvt_null_date_string);
 	}
 
 	/* Allow override of odbcinst.ini parameters here */
@@ -895,7 +904,7 @@ writeDriverCommoninfo(const char *fileName, const char *sectionName,
 	/*
 	 * Never update the onlyread from this module.
 	 */
-	if (ODBCINST_INI == fileName)
+	if (stricmp(ODBCINST_INI, fileName) == 0)
 	{
 		sprintf(tmp, "%d", comval->onlyread);
 		SQLWritePrivateProfileString(sectionName, INI_READONLY, tmp,
@@ -982,6 +991,7 @@ writeDSNinfo(const ConnInfo *ci)
 								 INI_USER,
 								 ci->username,
 								 ODBC_INI);
+	SQLWritePrivateProfileString(DSN, INI_UID, ci->username, ODBC_INI);
 
 	SQLWritePrivateProfileString(DSN,
 								 INI_PASSWORD,
@@ -1049,7 +1059,7 @@ writeDSNinfo(const ConnInfo *ci)
 								 INI_INT8AS,
 								 temp,
 								 ODBC_INI);
-	sprintf(temp, "%lx", getExtraOptions(ci));
+	sprintf(temp, "%x", getExtraOptions(ci));
 	SQLWritePrivateProfileString(DSN,
 							INI_EXTRAOPTIONS,
 							 temp,
@@ -1089,6 +1099,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 {
 	char		temp[256];
 	GLOBAL_VALUES *comval;
+	BOOL	inst_position = (stricmp(filename, ODBCINST_INI) == 0);
 
 	if (ci)
 		comval = &(ci->drivers);
@@ -1104,7 +1115,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 		if (comval->fetch_max <= 0)
 			comval->fetch_max = FETCH_MAX;
 	}
-	else if (!ci)
+	else if (inst_position)
 		comval->fetch_max = FETCH_MAX;
 
 	/* Socket Buffersize is stored in driver section */
@@ -1112,7 +1123,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->socket_buffersize = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->socket_buffersize = SOCK_BUFFER_SIZE;
 
 	/* Debug is stored in the driver section */
@@ -1120,7 +1131,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->debug = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->debug = DEFAULT_DEBUG;
 
 	/* CommLog is stored in the driver section */
@@ -1128,7 +1139,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->commlog = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->commlog = DEFAULT_COMMLOG;
 
 	if (!ci)
@@ -1138,7 +1149,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->disable_optimizer = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->disable_optimizer = DEFAULT_OPTIMIZER;
 
 	/* KSQO is stored in the driver section only */
@@ -1146,7 +1157,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->ksqo = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->ksqo = DEFAULT_KSQO;
 
 	/* Recognize Unique Index is stored in the driver section only */
@@ -1154,7 +1165,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->unique_index = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->unique_index = DEFAULT_UNIQUEINDEX;
 
 
@@ -1163,7 +1174,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->unknown_sizes = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->unknown_sizes = DEFAULT_UNKNOWNSIZES;
 
 
@@ -1172,7 +1183,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->lie = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->lie = DEFAULT_LIE;
 
 	/* Parse statements */
@@ -1180,7 +1191,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->parse = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->parse = DEFAULT_PARSE;
 
 	/* SQLCancel calls SQLFreeStmt in Driver Manager */
@@ -1188,7 +1199,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->cancel_as_freestmt = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->cancel_as_freestmt = DEFAULT_CANCELASFREESTMT;
 
 	/* UseDeclareFetch is stored in the driver section only */
@@ -1196,7 +1207,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->use_declarefetch = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->use_declarefetch = DEFAULT_USEDECLAREFETCH;
 
 	/* Max Varchar Size */
@@ -1204,7 +1215,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->max_varchar_size = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->max_varchar_size = MAX_VARCHAR_SIZE;
 
 	/* Max TextField Size */
@@ -1212,7 +1223,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->max_longvarchar_size = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->max_longvarchar_size = TEXT_FIELD_SIZE;
 
 	/* Text As LongVarchar	*/
@@ -1220,7 +1231,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->text_as_longvarchar = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->text_as_longvarchar = DEFAULT_TEXTASLONGVARCHAR;
 
 	/* Unknowns As LongVarchar	*/
@@ -1228,7 +1239,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->unknowns_as_longvarchar = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->unknowns_as_longvarchar = DEFAULT_UNKNOWNSASLONGVARCHAR;
 
 	/* Bools As Char */
@@ -1236,7 +1247,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (temp[0])
 		comval->bools_as_char = atoi(temp);
-	else if (!ci)
+	else if (inst_position)
 		comval->bools_as_char = DEFAULT_BOOLSASCHAR;
 
 	/* Extra Systable prefixes */
@@ -1249,14 +1260,14 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 							   temp, sizeof(temp), filename);
 	if (strcmp(temp, "@@@"))
 		strcpy(comval->extra_systable_prefixes, temp);
-	else if (!ci)
+	else if (inst_position)
 		strcpy(comval->extra_systable_prefixes, DEFAULT_EXTRASYSTABLEPREFIXES);
 
 	mylog("globals.extra_systable_prefixes = '%s'\n", comval->extra_systable_prefixes);
 
 
 	/* Dont allow override of an override! */
-	if (!ci)
+	if (inst_position)
 	{
 		/*
 		 * ConnSettings is stored in the driver section and per datasource
@@ -1290,8 +1301,7 @@ getCommonDefaults(const char *section, const char *filename, ConnInfo *ci)
 static void
 encode(const char *in, char *out)
 {
-	unsigned int i,
-				ilen = strlen(in),
+	size_t i, ilen = strlen(in),
 				o = 0;
 
 	for (i = 0; i < ilen; i++)
@@ -1339,8 +1349,7 @@ conv_from_hex(const UCHAR *s)
 static void
 decode(const char *in, char *out)
 {
-	unsigned int i,
-				ilen = strlen(in),
+	size_t i, ilen = strlen(in),
 				o = 0;
 
 	for (i = 0; i < ilen; i++)
