@@ -63,7 +63,7 @@ PGAPI_AllocEnv(HENV FAR * phenv)
 		return SQL_ERROR;
 	}
 
-	mylog("** exit PGAPI_AllocEnv: phenv = %x **\n", *phenv);
+	mylog("** exit PGAPI_AllocEnv: phenv = %p **\n", *phenv);
 	return SQL_SUCCESS;
 }
 
@@ -74,7 +74,7 @@ PGAPI_FreeEnv(HENV henv)
 	CSTR func = "PGAPI_FreeEnv";
 	EnvironmentClass *env = (EnvironmentClass *) henv;
 
-	mylog("**** in PGAPI_FreeEnv: env = %x ** \n", env);
+	mylog("**** in PGAPI_FreeEnv: env = %p ** \n", env);
 
 	if (env && EN_Destructor(env))
 	{
@@ -100,7 +100,7 @@ pg_sqlstate_set(const EnvironmentClass *env, UCHAR *szSqlState, const UCHAR *ver
 PG_ErrorInfo	*ER_Constructor(SDWORD errnumber, const char *msg)
 {
 	PG_ErrorInfo	*error;
-	Int4		aladd, errsize;
+	ssize_t		aladd, errsize;
 
 	if (DESC_OK == errnumber)
 		return NULL;
@@ -119,7 +119,7 @@ PG_ErrorInfo	*ER_Constructor(SDWORD errnumber, const char *msg)
 	{
 		memset(error, 0, sizeof(PG_ErrorInfo));
 		error->status = errnumber;
-		error->errorsize = errsize;
+		error->errorsize = (Int4) errsize;
 		if (errsize > 0)
 			memcpy(error->__error_message, msg, errsize);
 		error->__error_message[aladd] = '\0';
@@ -261,9 +261,9 @@ PGAPI_ConnectError(	HDBC hdbc,
 	char		*msg;
 	int		status;
 	BOOL	once_again = FALSE;
-	SWORD		msglen;
+	ssize_t		msglen;
 
-	mylog("**** PGAPI_ConnectError: hdbc=%x <%d>\n", hdbc, cbErrorMsgMax);
+	mylog("**** PGAPI_ConnectError: hdbc=%p <%d>\n", hdbc, cbErrorMsgMax);
 	if (RecNumber != 1 && RecNumber != -1)
 		return SQL_NO_DATA_FOUND;
 	if (cbErrorMsgMax < 0)
@@ -285,7 +285,7 @@ PGAPI_ConnectError(	HDBC hdbc,
 	msglen = strlen(msg);
 	if (NULL != pcbErrorMsg)
 	{
-		*pcbErrorMsg = msglen;
+		*pcbErrorMsg = (SQLSMALLINT) msglen;
 		if (cbErrorMsgMax == 0)
 			once_again = TRUE;
 		else if (msglen >= cbErrorMsgMax)
@@ -359,6 +359,7 @@ PGAPI_ConnectError(	HDBC hdbc,
 				break;
 			case CONNECTION_COULD_NOT_SEND:
 			case CONNECTION_COULD_NOT_RECEIVE:
+			case CONNECTION_COMMUNICATION_ERROR:
 				pg_sqlstate_set(env, szSqlState, "08S01", "08S01");
 				break;
 			default:
@@ -392,7 +393,7 @@ PGAPI_EnvError(		HENV henv,
 	char		*msg;
 	int		status;
 
-	mylog("**** PGAPI_EnvError: henv=%x <%d>\n", henv, cbErrorMsgMax);
+	mylog("**** PGAPI_EnvError: henv=%p <%d>\n", henv, cbErrorMsgMax);
 	if (RecNumber != 1 && RecNumber != -1)
 		return SQL_NO_DATA_FOUND;
 	if (cbErrorMsgMax < 0)
@@ -453,7 +454,7 @@ PGAPI_Error(
 	RETCODE	ret;
 	UWORD	flag = PODBC_ALLOW_PARTIAL_EXTRACT | PODBC_ERROR_CLEAR;
 
-	mylog("**** PGAPI_Error: henv=%x, hdbc=%x hstmt=%d\n", henv, hdbc, hstmt);
+	mylog("**** PGAPI_Error: henv=%p, hdbc=%p hstmt=%d\n", henv, hdbc, hstmt);
 
 	if (cbErrorMsgMax < 0)
 		return SQL_ERROR;
@@ -508,7 +509,7 @@ EN_Destructor(EnvironmentClass *self)
 	int			lf;
 	char		rv = 1;
 
-	mylog("in EN_Destructor, self=%x\n", self);
+	mylog("in EN_Destructor, self=%p\n", self);
 
 	/*
 	 * the error messages are static strings distributed throughout the
@@ -558,7 +559,7 @@ EN_add_connection(EnvironmentClass *self, ConnectionClass *conn)
 {
 	int			i;
 
-	mylog("EN_add_connection: self = %x, conn = %x\n", self, conn);
+	mylog("EN_add_connection: self = %p, conn = %p\n", self, conn);
 
 	ENTER_CONNS_CS;
 	for (i = 0; i < MAX_CONNECTIONS; i++)
@@ -569,7 +570,7 @@ EN_add_connection(EnvironmentClass *self, ConnectionClass *conn)
 			conns[i] = conn;
 			LEAVE_CONNS_CS;
 
-			mylog("       added at i =%d, conn->henv = %x, conns[i]->henv = %x\n", i, conn->henv, conns[i]->henv);
+			mylog("       added at i =%d, conn->henv = %p, conns[i]->henv = %p\n", i, conn->henv, conns[i]->henv);
 
 			return TRUE;
 		}
