@@ -40,6 +40,7 @@ CSTR	libpq = "libpq";
 #endif /* WIN32 */
 
 #if defined(_MSC_DELAY_LOAD_IMPORT)
+static BOOL	loaded_libpq = FALSE, loaded_ssllib = FALSE;
 /*
  *	Load psqlodbc path based libpq dll.
  */
@@ -59,7 +60,7 @@ static HMODULE LIBPQ_load_from_psqlodbc_path()
 		if (strnicmp(szFileName, sysdir, strlen(sysdir)) != 0)
 		{
 			hmodule = LoadLibraryEx(szFileName, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
-			mylog("psqlodbc path based libpq loaded module=%x\n", hmodule);
+			mylog("psqlodbc path based libpq loaded module=%p\n", hmodule);
 		}
 	}
 	return hmodule;
@@ -84,7 +85,7 @@ DliErrorHook(unsigned	dliNotify,
 	int	i;
 	static const char * const libarray[] = {"libssl32", "ssleay32"};
 
-	mylog("DliErrorHook Notify=%d %x\n", dliNotify, pdli);
+	mylog("DliErrorHook Notify=%d %p\n", dliNotify, pdli);
 	switch (dliNotify)
 	{
 		case dliFailLoadLib:
@@ -115,7 +116,7 @@ DliErrorHook(unsigned	dliNotify,
 #endif /* SSL_DLL */
 
 typedef BOOL (WINAPI *UnloadFunc)(LPCSTR);
-void UnloadDelayLoadedDLLs(BOOL ssllibLoaded)
+static void UnloadDelayLoadedDLLs(BOOL ssllibLoaded)
 {
 	BOOL	success;
 #if (_MSC_VER < 1300) /* VC6 DELAYLOAD IMPORT */
@@ -133,8 +134,17 @@ void UnloadDelayLoadedDLLs(BOOL ssllibLoaded)
 	}
 	return;
 }
+void CleanupDelayLoadedDLLs(void)
+{
+	if (loaded_libpq)
+		UnloadDelayLoadedDLLs(loaded_ssllib);
+}
 #else
-void UnloadDelayLoadedDLLs(BOOL SSLLoaded)
+static void UnloadDelayLoadedDLLs(BOOL SSLLoaded)
+{
+	return;
+}
+void CleanupDelayLoadedDLLs(void)
 {
 	return;
 }
@@ -142,7 +152,7 @@ void UnloadDelayLoadedDLLs(BOOL SSLLoaded)
 
 void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 {
-	void *pqconn;
+	void *pqconn = NULL;
 	*libpqLoaded = TRUE;
 #if defined(_MSC_DELAY_LOAD_IMPORT)
 	__try {
@@ -156,6 +166,12 @@ void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 	__except ((GetExceptionCode() & 0xffff) == ERROR_MOD_NOT_FOUND ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) {
 		*libpqLoaded = FALSE;
 	}
+	if (*libpqLoaded)
+	{
+		loaded_libpq = TRUE;
+		if (PQgetssl(pqconn))
+			loaded_ssllib = TRUE;
+	}
 #else
 	pqconn = PQconnectdb(conninfo);
 #endif /* _MSC_DELAY_LOAD_IMPORT */
@@ -172,7 +188,7 @@ BOOL LIBPQ_check()
 	if (!(hmodule = LoadLibrary(libpq)))
 		/* Second try the driver's directory */
 		hmodule = LIBPQ_load_from_psqlodbc_path();
-	mylog("hmodule=%x\n", hmodule);
+	mylog("hmodule=%p\n", hmodule);
 	if (hmodule)
 		FreeLibrary(hmodule);
 	return (NULL != hmodule);
