@@ -17,7 +17,8 @@ extern "C" {
 #endif
 BOOL	LIBPQ_check(void);
 void	*CALL_PQconnectdb(const char *conninfo, BOOL *);
-void	UnloadDelayLoadedDLLs(BOOL);
+/* void	UnloadDelayLoadedDLLs(BOOL); */
+void	CleanupDelayLoadedDLLs(void);
 
 #ifdef	__cplusplus
 }
