@@ -35,14 +35,17 @@ void		generate_filename(const char *, const char *, char *);
 void
 generate_filename(const char *dirname, const char *prefix, char *filename)
 {
-	int			pid = 0;
+#ifdef	WIN32
+	int	pid = 0;
 
-#ifndef WIN32
+	pid = _getpid();
+#else
+	pid_t	pid = 0;
 	struct passwd *ptr = 0;
 
 	ptr = getpwuid(getuid());
-#endif
 	pid = getpid();
+#endif
 	if (dirname == 0 || filename == 0)
 		return;
 
@@ -112,42 +115,58 @@ logs_on_off(int cnopen, int mylog_onoff, int qlog_onoff)
 	LEAVE_MYLOG_CS;
 }
 
+#ifdef	WIN32
+#define	LOGGING_PROCESS_TIME
+#endif /* WIN32 */
+#ifdef	LOGGING_PROCESS_TIME
+#include <mmsystem.h>
+	static	DWORD	start_time = 0;
+#endif /* LOGGING_PROCESS_TIME */
 #ifdef MY_LOG
 static FILE *MLOGFP = NULL;
 void
-mylog(char *fmt,...)
+mylog(const char *fmt,...)
 {
 	va_list		args;
 	char		filebuf[80];
 
+	if (!mylog_on)	return;
 	ENTER_MYLOG_CS;
-	if (mylog_on)
-	{
-		va_start(args, fmt);
+#ifdef	LOGGING_PROCESS_TIME
+	if (!start_time)
+		start_time = timeGetTime();
+#endif /* LOGGING_PROCESS_TIME */
+	va_start(args, fmt);
 
-		if (!MLOGFP)
-		{
-			generate_filename(MYLOGDIR, MYLOGFILE, filebuf);
-			MLOGFP = fopen(filebuf, PG_BINARY_A);
-			if (MLOGFP)
-				setbuf(MLOGFP, NULL);
-		}
+	if (!MLOGFP)
+	{
+		generate_filename(MYLOGDIR, MYLOGFILE, filebuf);
+		MLOGFP = fopen(filebuf, PG_BINARY_A);
+		if (MLOGFP)
+			setbuf(MLOGFP, NULL);
+	}
 
 #ifdef	WIN_MULTITHREAD_SUPPORT
 #ifdef	WIN32
-		if (MLOGFP)
-			fprintf(MLOGFP, "[%d]", GetCurrentThreadId());
+	if (MLOGFP)
+#ifdef	LOGGING_PROCESS_TIME
+	{
+		DWORD	proc_time = timeGetTime() - start_time;
+		fprintf(MLOGFP, "[%u-%d.%03d]", GetCurrentThreadId(), proc_time / 1000, proc_time % 1000);
+	}
+#else
+		fprintf(MLOGFP, "[%u]", GetCurrentThreadId());
+#endif /* LOGGING_PROCESS_TIME */
 #endif /* WIN32 */
 #endif /* WIN_MULTITHREAD_SUPPORT */
 #if defined(POSIX_MULTITHREAD_SUPPORT)
-		if (MLOGFP)
-			fprintf(MLOGFP, "[%d]", pthread_self());
+	if (MLOGFP)
+		fprintf(MLOGFP, "[%lu]", pthread_self());
 #endif /* POSIX_MULTITHREAD_SUPPORT */
-		if (MLOGFP)
-			vfprintf(MLOGFP, fmt, args);
+	if (MLOGFP)
+		vfprintf(MLOGFP, fmt, args);
 
-		va_end(args);
-	}
+	va_end(args);
 	LEAVE_MYLOG_CS;
 }
 void
@@ -187,7 +206,7 @@ forcelog(const char *fmt,...)
 #endif /* WIN32 */
 #endif /* WIN_MULTITHREAD_SUPPORT */
 #if defined(POSIX_MULTITHREAD_SUPPORT)
-		fprintf(MLOGFP, "[%d]", pthread_self());
+		fprintf(MLOGFP, "[%u]", pthread_self());
 #endif /* POSIX_MULTITHREAD_SUPPORT */
 		vfprintf(MLOGFP, fmt, args);
 	}
@@ -200,6 +219,7 @@ static void mylog_initialize()
 }
 static void mylog_finalize()
 {
+	mylog_on = 0;
 	if (MLOGFP)
 	{
 		fclose(MLOGFP);
@@ -225,24 +245,23 @@ qlog(char *fmt,...)
 	va_list		args;
 	char		filebuf[80];
 
+	if (!qlog_on)	return;
+
 	ENTER_QLOG_CS;
-	if (qlog_on)
+	va_start(args, fmt);
+
+	if (!QLOGFP)
 	{
-		va_start(args, fmt);
-
-		if (!QLOGFP)
-		{
-			generate_filename(QLOGDIR, QLOGFILE, filebuf);
-			QLOGFP = fopen(filebuf, PG_BINARY_A);
-			if (QLOGFP)
-				setbuf(QLOGFP, NULL);
-		}
-
+		generate_filename(QLOGDIR, QLOGFILE, filebuf);
+		QLOGFP = fopen(filebuf, PG_BINARY_A);
 		if (QLOGFP)
-			vfprintf(QLOGFP, fmt, args);
-
-		va_end(args);
+			setbuf(QLOGFP, NULL);
 	}
+
+	if (QLOGFP)
+		vfprintf(QLOGFP, fmt, args);
+
+	va_end(args);
 	LEAVE_QLOG_CS;
 }
 static void qlog_initialize()
@@ -251,6 +270,7 @@ static void qlog_initialize()
 }
 static void qlog_finalize()
 {
+	qlog_on = 0;
 	if (QLOGFP)
 	{
 		fclose(QLOGFP);
