@@ -59,7 +59,7 @@ SQLAllocHandle(SQLSMALLINT HandleType,
 			ENTER_CONN_CS(conn);
 			ret = PGAPI_AllocDesc(InputHandle, OutputHandle);
 			LEAVE_CONN_CS(conn);
-inolog("OutputHandle=%x\n", *OutputHandle);
+inolog("OutputHandle=%p\n", *OutputHandle);
 			break;
 		default:
 			ret = SQL_ERROR;
@@ -117,7 +117,9 @@ SQLColAttribute(SQLHSTMT StatementHandle,
 			SQLPOINTER CharacterAttribute,
 			SQLSMALLINT BufferLength,
 			SQLSMALLINT *StringLength,
-#if defined(WITH_UNIXODBC) || defined(WIN32)
+#if defined(_WIN64)
+			SQLLEN *NumericAttribute
+#elif defined(WITH_UNIXODBC) || defined(WIN32)
 			SQLPOINTER NumericAttribute
 #else
 			SQLLEN *NumericAttribute
@@ -192,7 +194,7 @@ SQLFetchScroll(HSTMT StatementHandle,
 	RETCODE		ret = SQL_SUCCESS;
 	IRDFields	*irdopts = SC_get_IRDF(stmt);
 	SQLUSMALLINT *rowStatusArray = irdopts->rowStatusArray;
-	SQLINTEGER *pcRow = (SQLINTEGER *) irdopts->rowsFetched, bkmarkoff = 0;
+	SQLLEN *pcRow = irdopts->rowsFetched, bkmarkoff = 0;
 
 	mylog("[[%s]] %d,%d\n", func, FetchOrientation, FetchOffset);
 	ENTER_STMT_CS(stmt);
@@ -291,9 +293,10 @@ SQLGetDiagField(SQLSMALLINT HandleType, SQLHANDLE Handle,
 				PTR DiagInfo, SQLSMALLINT BufferLength,
 				SQLSMALLINT *StringLength)
 {
+	CSTR func = "SQLGetDiagField";
 	RETCODE	ret;
 
-	mylog("[[SQLGetDiagField]] Handle=(%u,%x) Rec=%d Id=%d\n", HandleType, Handle, RecNumber, DiagIdentifier);
+	mylog("[[%s]] Handle=(%u,%p) Rec=%d Id=%d info=(%p,%d)\n", func, HandleType, Handle, RecNumber, DiagIdentifier, DiagInfo, BufferLength);
 	ret = PGAPI_GetDiagField(HandleType, Handle, RecNumber, DiagIdentifier,
 				DiagInfo, BufferLength, StringLength);
 	return ret;
@@ -412,7 +415,7 @@ SQLSetDescField(SQLHDESC DescriptorHandle,
 {
 	RETCODE		ret;
 
-	mylog("[[SQLSetDescField]] h=%x rec=%d field=%d val=%x\n", DescriptorHandle, RecNumber, FieldIdentifier, Value);
+	mylog("[[SQLSetDescField]] h=%p rec=%d field=%d val=%p\n", DescriptorHandle, RecNumber, FieldIdentifier, Value);
 	ret = PGAPI_SetDescField(DescriptorHandle, RecNumber, FieldIdentifier,
 				Value, BufferLength);
 	return ret;
@@ -448,7 +451,7 @@ SQLSetEnvAttr(HENV EnvironmentHandle,
 	switch (Attribute)
 	{
 		case SQL_ATTR_CONNECTION_POOLING:
-			switch ((SQLUINTEGER) Value)
+			switch ((ULONG_PTR) Value)
 			{
 			 	case SQL_CP_OFF:
 					EN_unset_pooling(env);
@@ -505,7 +508,7 @@ SQLSetStmtAttr(HSTMT StatementHandle,
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	RETCODE	ret;
 
-	mylog("[[%s]] Handle=%x %d,%u\n", func, StatementHandle, Attribute, Value);
+	mylog("[[%s]] Handle=%p %d,%u\n", func, StatementHandle, Attribute, Value);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
@@ -646,7 +649,7 @@ SQLBulkOperations(HSTMT hstmt, SQLSMALLINT operation)
 	StatementClass	*stmt = (StatementClass *) hstmt;
 
 	ENTER_STMT_CS(stmt);
-	mylog("[[%s]] Handle=%x %d\n", func, hstmt, operation);
+	mylog("[[%s]] Handle=%p %d\n", func, hstmt, operation);
 	SC_clear_error(stmt);
 	StartRollbackState(stmt);
 	ret = PGAPI_BulkOperations(hstmt, operation);
