@@ -47,8 +47,11 @@ static	char	searchColInfo(COL_INFO *col_info, FIELD_INFO *fi);
 
 Int4 FI_precision(const FIELD_INFO *fi)
 {
+	OID	ftype;
+
 	if (!fi)	return -1;
-	switch (fi->type)
+	ftype = FI_type(fi);
+	switch (ftype)
 	{
 		case PG_TYPE_NUMERIC:
 			return fi->column_size;
@@ -60,8 +63,11 @@ Int4 FI_precision(const FIELD_INFO *fi)
 }
 Int4 FI_scale(const FIELD_INFO *fi)
 {
+	OID	ftype;
+
 	if (!fi)	return -1;
-	switch (fi->type)
+	ftype = FI_type(fi);
+	switch (ftype)
 	{
 		case PG_TYPE_NUMERIC:
 			return fi->decimal_digits;
@@ -121,6 +127,9 @@ getNextToken(
 			break;
 		/* Handle quoted stuff */
 		in_quote = in_dollar_quote = FALSE;
+		taglen = 0;
+		tag = NULL;
+		escape_in_literal = '\0';
 		if (out == 0)
 		{
 			qc = s[i];
@@ -129,7 +138,7 @@ getNextToken(
 				in_quote = in_dollar_quote = TRUE;
 				tag = s + i;
 				taglen = 1;
-				if (tagend = strchr(s + i + 1, dollar_quote))
+				if (tagend = strchr(s + i + 1, dollar_quote), NULL != tagend)
 					taglen = tagend - s - i + 1;
 				i += (taglen - 1);
 				encoded_position_shift(&encstr, taglen - 1);
@@ -268,7 +277,7 @@ inolog("getColInfo non-manual result\n");
 	fi->dquote = TRUE;
 	STR_TO_NAME(fi->column_name, QR_get_value_backend_row(col_info->result, k, COLUMNS_COLUMN_NAME));
 
-	fi->type = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_FIELD_TYPE));
+	fi->columntype = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_FIELD_TYPE));
 	fi->column_size = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_PRECISION));
 	fi->length = atoi(QR_get_value_backend_row(col_info->result, k, COLUMNS_LENGTH));
 	if (str = QR_get_value_backend_row(col_info->result, k, COLUMNS_SCALE), str)
@@ -446,7 +455,9 @@ static BOOL increaseNtab(StatementClass *stmt, const char *func)
 static void xxxxx(FIELD_INFO *fi, QResultClass *res, int i)
 {
 	STR_TO_NAME(fi->column_alias, QR_get_fieldname(res, i));
-	fi->type = QR_get_field_type(res, i);
+	fi->basetype = QR_get_field_type(res, i);
+	if (0 == fi->columntype)
+		fi->columntype = fi->basetype;
 	if (fi->attnum < 0)
 	{
 		fi->nullable = FALSE;
@@ -480,11 +491,12 @@ static BOOL ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
 	IRDFields	*irdflds = SC_get_IRDF(stmt);
 	COL_INFO	*col_info = NULL;
 	FIELD_INFO	**fi, *wfi;
-	Oid		reloid = 0;
+	OID		reloid = 0;
 	Int2		attid;
 	int		i, num_fields;
 	BOOL		fi_reuse, updatable;
 
+mylog("ColAttSet in\n");
 	if (rti)
 	{
 		if (reloid = rti->table_oid, 0 == reloid)
@@ -514,13 +526,14 @@ static BOOL ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
 		irdflds->nfields = num_fields;
 	}
 	updatable = rti ? TI_is_updatable(rti) : FALSE;
+mylog("updatable=%d tab=%d fields=%d", updatable, stmt->ntab, num_fields);
 	if (updatable)
 	{
 		if (1 != stmt->ntab)
 			updatable = FALSE;
 		else
 		{
-			Oid	greloid;
+			OID	greloid;
 
 			for (i = 0; i < num_fields; i++)
 			{
@@ -533,9 +546,10 @@ static BOOL ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
 			}
 		}
 	}
+mylog("->%d\n", updatable);
 	for (i = 0; i < num_fields; i++)
 	{
-		if (reloid == (Oid) QR_get_relid(res, i))
+		if (reloid == (OID) QR_get_relid(res, i))
 		{
 			if (wfi = fi[i], NULL == wfi)
 			{
@@ -553,6 +567,7 @@ static BOOL ColAttSet(StatementClass *stmt, TABLE_INFO *rti)
 			if (searchColInfo(col_info, wfi))
 			{
 				STR_TO_NAME(wfi->column_alias, QR_get_fieldname(res, i));
+				wfi->basetype = QR_get_field_type(res, i);
 				wfi->updatable = updatable;
 			}
 			else
@@ -655,16 +670,16 @@ COL_INFO **coli)
 	return TRUE; /* success */
 }
 
-BOOL getCOLIfromTI(const char *func, ConnectionClass *conn, StatementClass *stmt, const Oid reloid, TABLE_INFO **pti)
+BOOL getCOLIfromTI(const char *func, ConnectionClass *conn, StatementClass *stmt, const OID reloid, TABLE_INFO **pti)
 {
 	BOOL	colatt = FALSE, found = FALSE;
-	Oid	greloid = reloid;
+	OID	greloid = reloid;
 	TABLE_INFO	*wti = *pti;
 	COL_INFO	*coli;
 	HSTMT		hcol_stmt = NULL;
 	QResultClass	*res;
 
-inolog("getCOLIfromTI reloid=%u ti=%x\n", reloid, wti);
+inolog("getCOLIfromTI reloid=%u ti=%p\n", reloid, wti);
 	if (!conn)
 		conn = SC_get_conn(stmt);
 	if (!wti)	/* SQLColAttribute case */
@@ -691,7 +706,7 @@ inolog("before increaseNtab\n");
 		}
 		*pti = wti;
 	}
-inolog("fi=%x greloid=%d col_info=%x\n", wti, greloid, wti->col_info);
+inolog("fi=%p greloid=%d col_info=%p\n", wti, greloid, wti->col_info);
 	if (0 == greloid)
 		greloid = wti->table_oid;
 	if (NULL != wti->col_info)
@@ -811,7 +826,7 @@ inolog("fi=%x greloid=%d col_info=%x\n", wti, greloid, wti->col_info);
 					STR_TO_NAME(wti->table_name, 
 						QR_get_value_backend_row(res, 0, COLUMNS_TABLE_NAME));
 			}
-inolog("#2 %x->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->table_oid);
+inolog("#2 %p->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->table_oid);
 			/*
 			 * Store the table name and the SQLColumns result
 			 * structure
@@ -862,7 +877,7 @@ cleanup:
 				STR_TO_NAME(wti->table_name, 
 					QR_get_value_backend_row(res, 0, COLUMNS_TABLE_NAME));
 		}
-inolog("#1 %x->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->table_oid);
+inolog("#1 %p->table_name=%s(%u)\n", wti, PRINT_NAME(wti->table_name), wti->table_oid);
 		if (colatt /* SQLColAttribute case */
 		    && 0 == (wti->flags & TI_COLATTRIBUTE))
 		{
@@ -1149,7 +1164,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 				if (quote)
 				{
 					wfi->quote = TRUE;
-					wfi->column_size = strlen(token);
+					wfi->column_size = (int) strlen(token);
 				}
 				else if (numeric)
 				{
@@ -1362,7 +1377,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 		if (wfi->func || wfi->expr || wfi->numeric)
 		{
 			wfi->ti = NULL;
-			wfi->type = (Oid) 0;
+			wfi->columntype = wfi->basetype = (OID) 0;
 			parse = FALSE;
 			continue;
 		}
@@ -1374,10 +1389,10 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 			 * wfi->type = PG_TYPE_TEXT; wfi->column_size = 0; the
 			 * following may be better
 			 */
-			wfi->type = PG_TYPE_UNKNOWN;
+			wfi->basetype = PG_TYPE_UNKNOWN;
 			if (wfi->column_size == 0)
 			{
-				wfi->type = PG_TYPE_VARCHAR;
+				wfi->basetype = PG_TYPE_VARCHAR;
 				wfi->column_size = 254;
 			}
 			wfi->length = wfi->column_size;
@@ -1471,8 +1486,6 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 	for (i = 0; i < stmt->ntab; i++)
 	{
 		/* See if already got it */
-		char		found = FALSE;
-
 		wti = ti[i];
 
 		if (!getCOLIfromTI(func, NULL, stmt, 0, &wti))
@@ -1504,9 +1517,9 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 		else if (SAFE_NAME(wfi->column_name)[0] == '*')
 		{
 			char		do_all_tables;
-			int			total_cols,
-						cols;
-			int			increased_cols;
+			Int2			total_cols,
+						cols,
+						increased_cols;
 
 			mylog("expanding field %d\n", i);
 
@@ -1514,13 +1527,13 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 
 			if (wfi->ti)		/* The star represents only the qualified
 								 * table */
-				total_cols = QR_get_num_cached_tuples(wfi->ti->col_info->result);
+				total_cols = (Int2) QR_get_num_cached_tuples(wfi->ti->col_info->result);
 
 			else
 			{	/* The star represents all tables */
 				/* Calculate the total number of columns after expansion */
 				for (k = 0; k < stmt->ntab; k++)
-					total_cols += QR_get_num_cached_tuples(ti[k]->col_info->result);
+					total_cols += (Int2) QR_get_num_cached_tuples(ti[k]->col_info->result);
 			}
 			increased_cols = total_cols - 1;
 
@@ -1570,7 +1583,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 			{
 				TABLE_INFO *the_ti = do_all_tables ? ti[k] : fi[i]->ti;
 
-				cols = QR_get_num_cached_tuples(the_ti->col_info->result);
+				cols = (Int2) QR_get_num_cached_tuples(the_ti->col_info->result);
 
 				for (n = 0; n < cols; n++)
 				{
@@ -1650,7 +1663,7 @@ parse_statement(StatementClass *stmt, BOOL check_hasoids)
 	{
 		wfi = fi[i];
 		wfi->flag &= ~FIELD_PARSING;
-		if (0 != wfi->type)
+		if (0 != wfi->columntype || 0 != wfi->basetype)
 			wfi->flag |= FIELD_PARSED_OK;
 	}
 
