@@ -5,7 +5,7 @@
  *
  * Comments:		See "notice.txt" for copyright and license information.
  *
- * $Id: psqlodbc.h,v 1.106 2006/08/30 12:18:30 hinoue Exp $
+ * $Id: psqlodbc.h,v 1.113 2006/10/20 12:00:32 hinoue Exp $
  *
  */
 
@@ -63,6 +63,12 @@
 #define SQLLEN SQLINTEGER
 #define SQLULEN SQLUINTEGER
 #define SQLSETPOSIROW SQLUSMALLINT
+/* VC6 bypasses 64bit mode. */
+#define DWLP_USER DWL_USER
+#define ULONG_PTR ULONG
+#define LONG_PTR LONG
+#define SetWindowLongPtr(hdlg, DWLP_USER, lParam) SetWindowLong(hdlg, DWLP_USER, lParam)
+#define GetWindowLongPtr(hdlg, DWLP_USER) GetWindowLong(hdlg, DWLP_USER);
 #endif
 #else
 #include "iodbc.h"
@@ -88,6 +94,66 @@ extern "C" {
 #define UInt4 unsigned int
 #define Int2 short
 #define UInt2 unsigned short
+typedef	UInt4	OID;
+
+#ifdef	WIN32
+#define	ssize_t	SSIZE_T
+#define	FORMAT_SIZE_T	"%Iu"	/* size_t */
+#define	FORMAT_SSIZE_T	"%Id"	/* ssize_t */
+#define	FORMAT_INTEGER	"%ld"	/* SQLINTEGER */
+#define	FORMAT_UINTEGER	"%lu"	/* SQLUINTEGER */
+#ifdef	_WIN64
+#define	FORMAT_LEN	"%I64d" /* SQLLEN */
+#define	FORMAT_ULEN	"%I64u" /* SQLULEN */
+#define	FORMAT_LPTR	"%I64d" /* LONG_PTR */
+#define	FORMAT_ULPTR	"%I64u" /* ULONG_PTR */
+#else
+#define	FORMAT_LEN	"%ld"	/* SQLLEN */
+#define	FORMAT_ULEN	"%lu"	/* SQLULEN */
+#define	FORMAT_LPTR	"%ld"	/* LONG_PTR */
+#define	FORMAT_ULPTR	"%lu"	/* ULONG_PTR */
+#endif /* _WIN64 */
+#else
+#define	FORMAT_SIZE_T	"%xu"	/* size_t */	
+#define	FORMAT_SSIZE_T	"%xd"	/* ssize_t */
+#ifndef	HAVE_SSIZE_T
+typedef	long	ssize_t
+#endif /* HAVE_SSIZE_T */
+#if (SIZE_OF_VOID_P == SIZE_OF_LONG)
+typedef	long 	LONG_PTR;
+typedef	unsigned long 	ULONG_PTR;
+#define	FORMAT_LPTR	"%ld"	/* LONG_PTR */
+#define	FORMAT_ULPTR	"%lu"	/* ULONG_PTR */
+#elif defined (HAVE_LONG_LONG)
+typedef	long long LONG_PTR;
+typedef	unsigned long long ULONG_PTR;
+#define	FORMAT_LPTR	"%lld"	/* LONG_PTR */
+#define	FORMAT_ULPTR	"%llu"	/* ULONG_PTR */
+#else
+#error appropriate long pointer type not found 
+#endif /* SIZE_OF_VOID_P */
+#if (SIZE_OF_VOID_P == 8)
+#define	FORMAT_INTEGER	"%d"	/* SQLINTEGER */
+#define	FORMAT_UINTEGER	"%u"	/* SQLUINTEGER */
+#if defined(WITH_UNIXODBC) && !defined(BUILD_REAL_64_BIT_MODE)
+#define FORMAT_LEN	"%d"	/* SQLLEN */
+#define FORMAT_ULEN	"%u"	/* SQLULEN */
+#else
+#define FORMAT_LEN	"%ld"	/* SQLLEN */
+#define FORMAT_ULEN	"%lu"	/* SQLULEN */
+#endif /* WITH_UNIXODBC */
+#else
+#define	FORMAT_LEN	"%ld"	/* SQLLEN */
+#define	FORMAT_ULEN	"%lu"	/* SQLULEN */
+#define	FORMAT_INTEGER	"%ld"	/* SQLINTEGER */
+#define	FORMAT_UINTEGER	"%lu"	/* SQLUINTEGER */
+#endif /* SIZE_OF_VOID_P */
+#endif /* WIN32 */
+#define	CAST_PTR(type, ptr)	(type)((LONG_PTR)(ptr))
+#define	CAST_UPTR(type, ptr)	(type)((ULONG_PTR)(ptr))
+#ifndef	SQL_IS_LEN
+#define	SQL_IS_LEN	(-1000)
+#endif /* SQL_IS_LEN */
 
 #ifndef WIN32
 #if !defined(WITH_UNIXODBC) && !defined(WITH_IODBC)
@@ -100,10 +166,6 @@ typedef double SDOUBLE;
 #endif /* CALLBACK */
 #endif /* WIN32 */
 
-#ifndef LIBPQ_FE_H
-typedef	UInt4	Oid;
-#endif /* LIBPQ_FE_H */
-
 #ifndef WIN32
 #define stricmp strcasecmp
 #define strnicmp strncasecmp
@@ -170,7 +232,11 @@ typedef	UInt4	Oid;
 #define DRIVER_FILE_NAME			"PSQLODBC25.DLL"
 #endif   /* ODBCVER 0x0300 */
 #else
-#define DRIVER_FILE_NAME			"psqlodbc.so"
+#ifdef  UNICODE_SUPPORT
+#define DRIVER_FILE_NAME                        "psqlodbcw.so"
+#else
+#define DRIVER_FILE_NAME                        "psqlodbca.so"
+#endif
 #endif   /* WIN32 */
 
 #define	NULL_CATALOG_NAME				""
@@ -253,7 +319,7 @@ typedef	UInt4	Oid;
 #define PG63						"6.3"		/* "Protocol" key setting
 												 * to force postgres 6.3 */
 #define PG64						"6.4"
-#define PG74REJECTED					"rejected7.4"
+#define PG74REJECTED					"reject7.4"
 #define PG74						"7.4"
 
 typedef int	(*PQFUNC)();
@@ -306,23 +372,23 @@ typedef struct GlobalValues_
 
 typedef struct StatementOptions_
 {
-	int			maxRows;
-	int			maxLength;
-	int			keyset_size;
-	int			cursor_type;
-	int			scroll_concurrency;
-	int			retrieve_data;
-	int			use_bookmarks;
+	SQLLEN			maxRows;
+	SQLLEN			maxLength;
+	SQLLEN			keyset_size;
+	SQLUINTEGER		cursor_type;
+	SQLUINTEGER		scroll_concurrency;
+	SQLUINTEGER		retrieve_data;
+	SQLUINTEGER		use_bookmarks;
 	void			*bookmark_ptr;
 #if (ODBCVER >= 0x0300)
-	int			metadata_id;
+	SQLUINTEGER		metadata_id;
 #endif /* ODBCVER */
 } StatementOptions;
 
 /*	Used to pass extra query info to send_query */
 typedef struct QueryInfo_
 {
-	int		row_size;
+	SQLLEN		row_size;
 	QResultClass	*result_in;
 	const char	*cursor;
 } QueryInfo;
@@ -330,12 +396,12 @@ typedef struct QueryInfo_
 /*	Used to save the error information */
 typedef struct
 {
-        SDWORD  status;
-        SDWORD  errorsize;
-        SWORD   recsize;
-        SWORD   errorpos;
+        UInt4	status;
+        Int4	errorsize;
+        Int2	recsize;
+        Int2	errorpos;
         char    sqlstate[8];
-        Int4    diag_row_count;
+        SQLLEN	diag_row_count;
         char    __error_message[1];
 }       PG_ErrorInfo;
 PG_ErrorInfo	*ER_Constructor(SDWORD errornumber, const char *errormsg);
@@ -381,9 +447,9 @@ const pthread_mutexattr_t *getMutexAttr(void);
 #endif /* POSIX_THREADMUTEX_SUPPORT */
 #ifdef	UNICODE_SUPPORT
 #define WCLEN sizeof(SQLWCHAR)
-UInt4	ucs2strlen(const SQLWCHAR *ucs2str);
-char	*ucs2_to_utf8(const SQLWCHAR *ucs2str, Int4 ilen, Int4 *olen, BOOL tolower);
-UInt4	utf8_to_ucs2_lf(const char * utf8str, Int4 ilen, BOOL lfconv, SQLWCHAR *ucs2str, UInt4 buflen);
+SQLULEN	ucs2strlen(const SQLWCHAR *ucs2str);
+char	*ucs2_to_utf8(const SQLWCHAR *ucs2str, SQLLEN ilen, SQLLEN *olen, BOOL tolower);
+SQLULEN	utf8_to_ucs2_lf(const char * utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN buflen);
 #define	utf8_to_ucs2(utf8str, ilen, ucs2str, buflen) utf8_to_ucs2_lf(utf8str, ilen, FALSE, ucs2str, buflen)
 #endif /* UNICODE_SUPPORT */
 
