@@ -33,13 +33,26 @@
 
 extern GLOBAL_VALUES globals;
 
-#define	SOCK_set_error(s, _no, _msg)	(s->errornumber = _no, s->errormsg = _msg, mylog("socket error=%d %s\n", _no, _msg))
+static void SOCK_set_error(SocketClass *s, int _no, const char *_msg)
+{
+	int	gerrno = SOCK_ERRNO;
+	s->errornumber = _no;
+	if (NULL != s->_errormsg_)
+		free(s->_errormsg_);
+	if (NULL != _msg)
+		s->_errormsg_ = strdup(_msg);
+	else
+		s->_errormsg_ = NULL;
+	mylog("(%d)%s ERRNO=%d\n", _no, _msg, gerrno);
+}
 
 void
 SOCK_clear_error(SocketClass *self)
 {
 	self->errornumber = 0;
-	self->errormsg = NULL;
+	if (NULL != self->_errormsg_)
+		free(self->_errormsg_);
+	self->_errormsg_ = NULL;
 }
 
 
@@ -80,7 +93,7 @@ SOCK_Constructor(const ConnectionClass *conn)
 			free(rv);
 			return NULL;
 		}
-		rv->errormsg = NULL;
+		rv->_errormsg_ = NULL;
 		rv->errornumber = 0;
 		rv->reverse = FALSE;
 	}
@@ -94,7 +107,7 @@ SOCK_Destructor(SocketClass *self)
 	mylog("SOCK_Destructor\n");
 	if (!self)
 		return;
-	if (self->socket != -1)
+	if (self->socket != (SOCKETFD) -1)
 	{
 		if (self->pqconn)
 		{
@@ -102,7 +115,7 @@ SOCK_Destructor(SocketClass *self)
 			{
 				if (self->pqconn)
 					PQfinish(self->pqconn);
-				UnloadDelayLoadedDLLs(NULL != self->ssl);
+				/* UnloadDelayLoadedDLLs(NULL != self->ssl); */
 			}
 			self->via_libpq = FALSE;
 			self->pqconn = NULL;
@@ -123,6 +136,8 @@ SOCK_Destructor(SocketClass *self)
 
 	if (self->buffer_out)
 		free(self->buffer_out);
+	if (self->_errormsg_)
+		free(self->_errormsg_);
 
 	free(self);
 }
@@ -130,21 +145,65 @@ SOCK_Destructor(SocketClass *self)
 #if defined(_MSC_VER) && (_MSC_VER < 1300)
 static freeaddrinfo_func freeaddrinfo_ptr = NULL;
 static getaddrinfo_func getaddrinfo_ptr = NULL;
+static getnameinfo_func getnameinfo_ptr = NULL;
 static	HMODULE ws2_hnd = NULL;
 #else
 static freeaddrinfo_func freeaddrinfo_ptr = freeaddrinfo;
 static getaddrinfo_func getaddrinfo_ptr = getaddrinfo;
+static getnameinfo_func getnameinfo_ptr = getnameinfo;
 #endif /* _MSC_VER */
 
+static BOOL format_sockerr(char *errmsg, size_t buflen, int errnum, const char *cmd, const char *host, int portno)
+{
+	BOOL ret = FALSE;
+
+#ifdef	WIN32
+	if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL,
+		errnum, MAKELANGID(LANG_ENGLISH, SUBLANG_DEFAULT),
+		errmsg, (DWORD)buflen, NULL))
+		ret = TRUE;
+#else
+#if defined(POSIX_MULTITHREAD_SUPPORT) && defined(HAVE_STRERROR_R)
+#ifdef	STRERROR_R_INT
+	if (0 == strerror_r(errnum, errmsg, buflen))
+		ret = TRUE;
+#else
+	const char *pchar;
+
+	pchar = (const char *) strerror_r(errnum, errmsg, buflen);
+	if (NULL != pchar)
+	{
+		if (pchar != errmsg)
+			strncpy(errmsg, pchar, buflen);
+		ret = TRUE;
+	}
+#endif /* STRERROR_R_INT */
+#else
+	strncpy(errmsg, strerror(errnum), buflen);
+	ret = TRUE;
+#endif /* POSIX_MULTITHREAD_SUPPORT */
+#endif /* WIN32 */
+	if (ret)
+	{
+		size_t	tlen = strlen(errmsg);
+		errmsg += tlen;
+		buflen -= tlen;
+		snprintf(errmsg, buflen, " [%s:%d]", host, portno);
+	}
+	else
+		snprintf(errmsg, buflen, "%s failed for [%s:%d] ", cmd, host, portno);
+	return ret;
+}
+ 
 char
 SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long timeout)
 {
 	struct addrinfo	rest, *addrs = NULL, *curadr = NULL;
-	int	family; 
+	int	family = 0; 
 	char	retval = 0;
 
 
-	if (self->socket != -1)
+	if (self->socket != (SOCKETFD) -1)
 	{
 		SOCK_set_error(self, SOCKET_ALREADY_CONNECTED, "Socket is already connected");
 		return 0;
@@ -155,6 +214,10 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long tim
 		ws2_hnd = GetModuleHandle("ws2_32.dll");
 	if (freeaddrinfo_ptr == NULL)
 		freeaddrinfo_ptr = (freeaddrinfo_func)GetProcAddress(ws2_hnd, "freeaddrinfo"); 
+	if (getaddrinfo_ptr == NULL)
+		getaddrinfo_ptr = (getaddrinfo_func)GetProcAddress(ws2_hnd, "getaddrinfo"); 
+	if (getnameinfo_ptr == NULL)
+		getnameinfo_ptr = (getnameinfo_func)GetProcAddress(ws2_hnd, "getnameinfo"); 
 #endif
 	/*
 	 * If it is a valid IP address, use it. Otherwise use hostname lookup.
@@ -169,7 +232,7 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long tim
 		rest.ai_family = AF_UNSPEC;
 		snprintf(portstr, sizeof(portstr), "%d", port);
 		if (inet_addr(hostname) != INADDR_NONE)
-			rest.ai_flags = AI_NUMERICHOST;	
+			rest.ai_flags |= AI_NUMERICHOST;	
 		ret = getaddrinfo_ptr(hostname, portstr, &rest, &addrs);
 		if (ret || !addrs)
 		{
@@ -200,7 +263,7 @@ retry:
 	if (curadr)
 		family = curadr->ai_family;
 	self->socket = socket(family, SOCK_STREAM, 0);
-	if (self->socket == -1)
+	if (self->socket == (SOCKETFD) -1)
 	{
 		SOCK_set_error(self, SOCKET_COULD_NOT_CREATE_SOCKET, "Could not create Socket.");
 		return 0;
@@ -238,15 +301,15 @@ retry:
 		struct sockaddr *in = (struct sockaddr *) &(self->sadr_area);
 		memset((char *) in, 0, sizeof(self->sadr_area));
 		memcpy(in, curadr->ai_addr, curadr->ai_addrlen);
-		self->sadr_len = curadr->ai_addrlen;
+		self->sadr_len = (int) curadr->ai_addrlen;
 	}
 	if (connect(self->socket, (struct sockaddr *) &(self->sadr_area), self->sadr_len) < 0)
 	{
 		int	ret, optval;
-		fd_set	fds;
+		fd_set	fds, except_fds;
 		struct	timeval	tm;
 		socklen_t	optlen = sizeof(optval);
-		time_t	t_now, t_finish;
+		time_t	t_now, t_finish = 0;
 		BOOL	tm_exp = FALSE;
 
 		switch (SOCK_ERRNO)
@@ -257,8 +320,7 @@ retry:
 			case EWOULDBLOCK:
 		    		break;
 			default:
-				SOCK_set_error(self, SOCKET_COULD_NOT_CONNECT, "Could not connect to remote socket ..");
-				mylog("connect immediate connect error %d\n", SOCK_ERRNO);
+				SOCK_set_error(self, SOCKET_COULD_NOT_CONNECT, "Could not connect to remote socket immedaitely");
 				goto cleanup;
 		}
 		if (timeout > 0)
@@ -270,8 +332,10 @@ retry:
 		}
 		do {
 			FD_ZERO(&fds);
+			FD_ZERO(&except_fds);
 			FD_SET(self->socket, &fds);
-			ret = select(self->socket + 1, NULL, &fds, NULL, timeout > 0 ? &tm : NULL);
+			FD_SET(self->socket, &except_fds);
+			ret = select((int) self->socket + 1, NULL, &fds, &except_fds, timeout > 0 ? &tm : NULL);
 			if (0 < ret)
 				break;
 			else if (0 == ret)
@@ -284,7 +348,7 @@ retry:
 					tm_exp = TRUE;
 				else
 				{
-					tm.tv_sec = t_finish - t_now;
+					tm.tv_sec = (long) (t_finish - t_now);
 					tm.tv_usec = 0;
 				}
 			}
@@ -307,8 +371,16 @@ retry:
 		}
 		else if (optval != 0)
 		{
-			SOCK_set_error(self, SOCKET_COULD_NOT_CONNECT, "Could not connect to remote server.");
-			mylog("connect getsockopt val=%d\n", optval);
+			char	errmsg[256], host[64];
+
+			host[0] = '\0';
+			getnameinfo_ptr((struct sockaddr *) &(self->sadr_area),
+					self->sadr_len, host, sizeof(host),
+					NULL, 0, NI_NUMERICHOST);
+			/* snprintf(errmsg, sizeof(errmsg), "connect getsockopt val %d addr=%s\n", optval, host); */
+			format_sockerr(errmsg, sizeof(errmsg), optval, "connect", host, port);
+			mylog(errmsg);
+			SOCK_set_error(self, SOCKET_COULD_NOT_CONNECT, errmsg);
 		}
 		else
 			retval = 1;
@@ -329,9 +401,12 @@ cleanup:
 			curadr = curadr->ai_next;
 			goto retry;
 		}
-		if (addrs)
-			freeaddrinfo_ptr(addrs);
 	}
+	else
+		SOCK_set_error(self, 0, NULL);
+	
+	if (addrs)
+		freeaddrinfo_ptr(addrs);
 	return retval;
 }
 
@@ -339,23 +414,28 @@ cleanup:
 /*
  *	To handle EWOULDBLOCK etc (mainly for libpq non-blocking connection).
  */
-#define	MAX_RETRY_COUNT	8
+#define	MAX_RETRY_COUNT	30
 static int SOCK_wait_for_ready(SocketClass *sock, BOOL output, int retry_count)
 {
 	int	ret;
-	fd_set	fds;
+	fd_set	fds, except_fds;
 	struct	timeval	tm;
+	BOOL	no_timeout = (0 != retry_count && (retry_count < 0 || (!sock->ssl)));
 
 	do {
 		FD_ZERO(&fds);
+		FD_ZERO(&except_fds);
 		FD_SET(sock->socket, &fds);
-		if (!output && sock->ssl)
+		FD_SET(sock->socket, &except_fds);
+		if (!no_timeout)
 		{
 			tm.tv_sec = retry_count;
 			tm.tv_usec = 0;
 		}
-		ret = select(sock->socket + 1, output ? NULL : &fds, output ? &fds : NULL, NULL, sock->ssl ? &tm : NULL);
+		ret = select((int)sock->socket + 1, output ? NULL : &fds, output ? &fds : NULL, &except_fds, no_timeout ? NULL : &tm);
 	} while (ret < 0 && EINTR == SOCK_ERRNO);
+	if (retry_count < 0)
+		retry_count *= -1;
 	if (0 == ret && retry_count > MAX_RETRY_COUNT)
 	{
 		ret = -1;
@@ -470,12 +550,16 @@ SOCK_get_id(SocketClass *self)
 {
 	int	id;
 
+	if (0 != self->errornumber)
+		return 0;
 	if (self->reslen > 0)
 	{
 		mylog("SOCK_get_id has to eat %d bytes\n", self->reslen);
 		do
 		{
 			SOCK_get_next_byte(self);
+			if (0 != self->errornumber)
+				return 0;
 		} while (self->reslen > 0);
 	}
 	id = SOCK_get_next_byte(self);
@@ -484,7 +568,7 @@ SOCK_get_id(SocketClass *self)
 }
 
 void
-SOCK_get_n_char(SocketClass *self, char *buffer, int len)
+SOCK_get_n_char(SocketClass *self, char *buffer, Int4 len)
 {
 	int			lf;
 
@@ -497,12 +581,16 @@ SOCK_get_n_char(SocketClass *self, char *buffer, int len)
 	}
 
 	for (lf = 0; lf < len; lf++)
+	{
+		if (0 != self->errornumber)
+			break;
 		buffer[lf] = SOCK_get_next_byte(self);
+	}
 }
 
 
 void
-SOCK_put_n_char(SocketClass *self, char *buffer, int len)
+SOCK_put_n_char(SocketClass *self, char *buffer, Int4 len)
 {
 	int			lf;
 
@@ -515,7 +603,11 @@ SOCK_put_n_char(SocketClass *self, char *buffer, int len)
 	}
 
 	for (lf = 0; lf < len; lf++)
+	{
+		if (0 != self->errornumber)
+			break;
 		SOCK_put_next_byte(self, (UCHAR) buffer[lf]);
+	}
 }
 
 
@@ -525,9 +617,9 @@ SOCK_put_n_char(SocketClass *self, char *buffer, int len)
  *	returns TRUE if truncation occurs.
  */
 BOOL
-SOCK_get_string(SocketClass *self, char *buffer, int bufsize)
+SOCK_get_string(SocketClass *self, char *buffer, Int4 bufsize)
 {
-	register int lf = 0;
+	int lf;
 
 	for (lf = 0; lf < bufsize - 1; lf++)
 		if (!(buffer[lf] = SOCK_get_next_byte(self)))
@@ -541,13 +633,16 @@ SOCK_get_string(SocketClass *self, char *buffer, int bufsize)
 void
 SOCK_put_string(SocketClass *self, const char *string)
 {
-	register int lf;
-	int			len;
+	size_t	lf, len;
 
 	len = strlen(string) + 1;
 
 	for (lf = 0; lf < len; lf++)
+	{
+		if (0 != self->errornumber)
+			break;
 		SOCK_put_next_byte(self, (UCHAR) string[lf]);
+	}
 }
 
 
@@ -613,14 +708,16 @@ SOCK_put_int(SocketClass *self, int value, short len)
 }
 
 
-void
+Int4
 SOCK_flush_output(SocketClass *self)
 {
-	int	written, pos = 0, retry_count = 0;
+	int	written, pos = 0, retry_count = 0, ttlsnd = 0;
 
 	if (!self)
-		return;
-	do
+		return -1;
+	if (0 != self->errornumber)
+		return -1;
+	while (self->buffer_filled_out > 0)
 	{
 		if (self->ssl)
 			written = SOCK_SSL_send(self, (char *) self->buffer_out + pos, self->buffer_filled_out);
@@ -640,11 +737,15 @@ SOCK_flush_output(SocketClass *self)
 					break;
 			}
 			SOCK_set_error(self, SOCKET_WRITE_ERROR, "Could not flush socket buffer.");
-			break;
+			return -1;
 		}
 		pos += written;
 		self->buffer_filled_out -= written;
-	} while (self->buffer_filled_out > 0);
+		ttlsnd += written;
+		retry_count = 0;
+	}
+	
+	return ttlsnd;
 }
 
 
@@ -652,6 +753,7 @@ UCHAR
 SOCK_get_next_byte(SocketClass *self)
 {
 	int	retry_count = 0;
+	BOOL	maybeEOF = FALSE;
 
 	if (!self)
 		return 0;
@@ -682,6 +784,11 @@ mylog("Lasterror=%d\n", SOCK_ERRNO);
 					if (SOCK_wait_for_ready(self, FALSE, retry_count) >= 0)
 						goto retry;
 					break;
+				case	ECONNRESET:
+inolog("ECONNRESET\n");
+					maybeEOF = TRUE;
+					SOCK_set_error(self, SOCKET_CLOSED, "Connection reset by peer.");
+					break;
 			}
 			if (0 == self->errornumber)
 				SOCK_set_error(self, SOCKET_READ_ERROR, "Error while reading from the socket.");
@@ -690,8 +797,21 @@ mylog("Lasterror=%d\n", SOCK_ERRNO);
 		}
 		if (self->buffer_filled_in == 0)
 		{
-			SOCK_set_error(self, SOCKET_CLOSED, "Socket has been closed.");
-			self->buffer_filled_in = 0;
+			if (!maybeEOF)
+			{
+				int	nbytes = SOCK_wait_for_ready(self, FALSE, 0);
+				if (nbytes > 0)
+				{
+					maybeEOF = TRUE;
+					goto retry;
+				}
+				else if (0 == nbytes)
+					maybeEOF = TRUE;
+			}
+			if (maybeEOF)
+				SOCK_set_error(self, SOCKET_CLOSED, "Socket has been closed.");
+			else
+				SOCK_set_error(self, SOCKET_READ_ERROR, "Error while reading from the socket.");
 			return 0;
 		}
 	}
@@ -707,6 +827,8 @@ SOCK_put_next_byte(SocketClass *self, UCHAR next_byte)
 
 	if (!self)
 		return;
+	if (0 != self->errornumber)
+		return;
 	self->buffer_out[self->buffer_filled_out++] = next_byte;
 
 	if (self->buffer_filled_out == self->buffer_size)
@@ -735,11 +857,12 @@ SOCK_put_next_byte(SocketClass *self, UCHAR next_byte)
 			}
 			pos += bytes_sent;
 			self->buffer_filled_out -= bytes_sent;
+			retry_count = 0;
 		} while (self->buffer_filled_out > 0);
 	}
 }
 
-int
+Int4
 SOCK_get_response_length(SocketClass *self)
 {
 	int     leng = -1;
