@@ -55,6 +55,8 @@ typedef unsigned int in_addr_t;
 #define AI_CANONNAME   0x2  // Return canonical name in first ai_canonname.
 #define AI_NUMERICHOST 0x4  // Nodename must be a numeric address string.
 
+#define NI_NUMERICHOST	1
+
 #define _SS_MAXSIZE 128
 #define _SS_ALIGNSIZE (sizeof(__int64))
 #define _SS_PAD1SIZE (_SS_ALIGNSIZE - sizeof (short))
@@ -98,7 +100,7 @@ typedef void (WSAAPI *freeaddrinfo_func) (struct addrinfo *);
 typedef int (WSAAPI *getaddrinfo_func) (const char *, const char *,
 	const struct addrinfo *, struct addrinfo **); 
 typedef int (WSAAPI *getnameinfo_func) (const struct sockaddr *,
-	socklen_t, char *, DWORD, char *, DWORD, int);
+	socklen_t, char *, size_t, char *, size_t, int);
 
 #define SOCKET_ALREADY_CONNECTED		1
 #define SOCKET_HOST_NOT_FOUND			2
@@ -128,7 +130,7 @@ struct SocketClass_
 	unsigned int	pversion;
 	int		reslen;
 
-	char		*errormsg;
+	char		*_errormsg_;
 	int		errornumber;
 	int		sadr_len;
 	struct sockaddr_storage sadr_area; /* Used for various connections */
@@ -148,7 +150,7 @@ struct SocketClass_
 
 /* error functions */
 #define SOCK_get_errcode(self)	(self ? self->errornumber : SOCKET_CLOSED)
-#define SOCK_get_errmsg(self)	(self ? self->errormsg : "socket closed")
+#define SOCK_get_errmsg(self)	(self ? self->_errormsg_ : "socket closed")
 
 /*
  *	code taken from postgres libpq et al.
@@ -186,16 +188,16 @@ SocketClass *SOCK_Constructor(const ConnectionClass *conn);
 void		SOCK_Destructor(SocketClass *self);
 char		SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long timeout);
 int		SOCK_get_id(SocketClass *self);
-void		SOCK_get_n_char(SocketClass *self, char *buffer, int len);
-void		SOCK_put_n_char(SocketClass *self, char *buffer, int len);
-BOOL		SOCK_get_string(SocketClass *self, char *buffer, int bufsize);
+void		SOCK_get_n_char(SocketClass *self, char *buffer, Int4 len);
+void		SOCK_put_n_char(SocketClass *self, char *buffer, Int4 len);
+BOOL		SOCK_get_string(SocketClass *self, char *buffer, Int4 bufsize);
 void		SOCK_put_string(SocketClass *self, const char *string);
-int			SOCK_get_int(SocketClass *self, short len);
+int		SOCK_get_int(SocketClass *self, short len);
 void		SOCK_put_int(SocketClass *self, int value, short len);
-void		SOCK_flush_output(SocketClass *self);
+Int4		SOCK_flush_output(SocketClass *self);
 UCHAR		SOCK_get_next_byte(SocketClass *self);
 void		SOCK_put_next_byte(SocketClass *self, UCHAR next_byte);
-int		SOCK_get_response_length(SocketClass *self);
+Int4		SOCK_get_response_length(SocketClass *self);
 void		SOCK_clear_error(SocketClass *self);
 UInt4		SOCK_skip_n_bytes(SocketClass *self, UInt4 skip_length);
 
