@@ -175,7 +175,7 @@ PGAPI_AllocStmt(HDBC hdbc,
 
 	stmt = SC_Constructor(conn);
 
-	mylog("**** PGAPI_AllocStmt: hdbc = %x, stmt = %x\n", hdbc, stmt);
+	mylog("**** PGAPI_AllocStmt: hdbc = %p, stmt = %p\n", hdbc, stmt);
 
 	if (!stmt)
 	{
@@ -215,7 +215,7 @@ PGAPI_FreeStmt(HSTMT hstmt,
 	CSTR func = "PGAPI_FreeStmt";
 	StatementClass *stmt = (StatementClass *) hstmt;
 
-	mylog("%s: entering...hstmt=%x, fOption=%d\n", func, hstmt, fOption);
+	mylog("%s: entering...hstmt=%p, fOption=%hi\n", func, hstmt, fOption);
 
 	if (!stmt)
 	{
@@ -423,7 +423,7 @@ SC_Destructor(StatementClass *self)
 	QResultClass	*res = SC_get_Result(self);
 
 	if (!self)	return FALSE;
-	mylog("SC_Destructor: self=%x, self->result=%x, self->hdbc=%x\n", self, res, self->hdbc);
+	mylog("SC_Destructor: self=%p, self->result=%p, self->hdbc=%p\n", self, res, self->hdbc);
 	SC_clear_error(self);
 	if (STMT_EXECUTING == self->status)
 	{
@@ -532,9 +532,9 @@ void
 SC_set_rowset_start(StatementClass *stmt, SQLLEN start, BOOL valid_base)
 {
 	QResultClass	*res = SC_get_Curres(stmt);
-	Int4	incr = start - stmt->rowset_start;
+	SQLLEN		incr = start - stmt->rowset_start;
 
-inolog("%x->SC_set_rowstart %d->%d(%s) ", stmt, stmt->rowset_start, start, valid_base ? "valid" : "unknown");
+inolog("%p->SC_set_rowstart " FORMAT_LEN "->" FORMAT_LEN "(%s) ", stmt, stmt->rowset_start, start, valid_base ? "valid" : "unknown");
 	if (res != NULL)
 	{
 		BOOL	valid = QR_has_valid_base(res);
@@ -557,10 +557,10 @@ inolog(":QR is %s", QR_has_valid_base(res) ? "valid" : "unknown");
 		}
 		if (!QR_get_cursor(res))
 			res->key_base = start;
-inolog(":QR result=%d(%s)", QR_get_rowstart_in_cache(res), QR_has_valid_base(res) ? "valid" : "unknown");
+inolog(":QR result=" FORMAT_LEN "(%s)", QR_get_rowstart_in_cache(res), QR_has_valid_base(res) ? "valid" : "unknown");
 	}
 	stmt->rowset_start = start;
-inolog(":stmt result=%d\n", stmt->rowset_start);
+inolog(":stmt result=" FORMAT_LEN "\n", stmt->rowset_start);
 }
 void
 SC_inc_rowset_start(StatementClass *stmt, SQLLEN inc)
@@ -717,7 +717,7 @@ SC_recycle_statement(StatementClass *self)
 	ConnectionClass *conn;
 	QResultClass	*res;
 
-	mylog("%s: self= %x\n", func, self);
+	mylog("%s: self= %p\n", func, self);
 
 	SC_clear_error(self);
 	/* This would not happen */
@@ -759,22 +759,25 @@ SC_recycle_statement(StatementClass *self)
 			return FALSE;
 	}
 
-	if (NOT_YET_PREPARED == self->prepared)
+	switch (self->prepared)
 	{
-        	/* Free the parsed table information */
-		if (self->ti)
-		{
-			TI_Destructor(self->ti, self->ntab);
-			free(self->ti);
-			self->ti = NULL;
-			self->ntab = 0;
-		}
-		/* Free the parsed field information */
-		DC_Destructor((DescriptorClass *) SC_get_IRD(self));
+		case NOT_YET_PREPARED:
+		case ONCE_DESCRIBED:
+        		/* Free the parsed table information */
+			if (self->ti)
+			{
+				TI_Destructor(self->ti, self->ntab);
+				free(self->ti);
+				self->ti = NULL;
+				self->ntab = 0;
+			}
+			/* Free the parsed field information */
+			DC_Destructor((DescriptorClass *) SC_get_IRD(self));
 
 inolog("SC_clear_parse_status\n");
-		SC_clear_parse_status(self, conn);
-		self->updatable = FALSE;
+			SC_clear_parse_status(self, conn);
+			self->updatable = FALSE;
+			break;
 	}
 
 	/* Free any cursors */
@@ -802,7 +805,7 @@ inolog("SC_clear_parse_status\n");
 	SC_set_rowset_start(self, -1, FALSE);
 	SC_set_current_col(self, -1);
 	self->bind_row = 0;
-inolog("%s statement=%x ommitted=0\n", func, self);
+inolog("%s statement=%p ommitted=0\n", func, self);
 	self->last_fetch_count = self->last_fetch_count_include_ommitted = 0;
 
 	self->__error_message = NULL;
@@ -854,15 +857,16 @@ SC_pre_execute(StatementClass *self)
 		{
 			char		old_pre_executing = self->pre_executing;
 
-			decideHowToPrepare(self);
+			decideHowToPrepare(self, FALSE);
 			self->inaccurate_result = FALSE;
 			switch (SC_get_prepare_method(self))
 			{
-				case USING_PARSE_REQUEST:
+				case NAMED_PARSE_REQUEST:
+				case PARSE_TO_EXEC_ONCE:
 					if (SQL_SUCCESS != prepareParameters(self))
 						return num_fields;
 					break;
-				case USING_UNNAMED_PARSE_REQUEST:
+				case PARSE_REQ_FOR_INFO:
 					if (SQL_SUCCESS != prepareParameters(self))
 						return num_fields;
 					self->status = STMT_PREMATURE;
@@ -1011,7 +1015,7 @@ SC_create_errorinfo(const StatementClass *self)
 	QResultClass *res = SC_get_Curres(self);
 	ConnectionClass *conn = SC_get_conn(self);
 	Int4	errornum;
-	int		pos;
+	size_t		pos;
 	BOOL		resmsg = FALSE, detailmsg = FALSE, msgend = FALSE;
 	char		msg[4096], *wmsg;
 	char		*ermsg = NULL, *sqlstate = NULL;
@@ -1038,7 +1042,7 @@ SC_create_errorinfo(const StatementClass *self)
 		else if (QR_get_notice(res))
 		{
 			char *notice = QR_get_notice(res);
-			int len = strlen(notice);
+			size_t	len = strlen(notice);
 			if (len < sizeof(msg))
 			{
 				memcpy(msg, notice, len);
@@ -1071,6 +1075,7 @@ SC_create_errorinfo(const StatementClass *self)
 	if (conn && !msgend)
 	{
 		SocketClass *sock = conn->sock;
+		const char *sockerrmsg;
 
 		if (!resmsg && (wmsg = CC_get_errormsg(conn)) && wmsg[0] != '\0')
 		{
@@ -1078,10 +1083,10 @@ SC_create_errorinfo(const StatementClass *self)
 			snprintf(&msg[pos], sizeof(msg) - pos, ";\n%s", CC_get_errormsg(conn));
 		}
 
-		if (sock && sock->errormsg && sock->errormsg[0] != '\0')
+		if (sock && NULL != (sockerrmsg = SOCK_get_errmsg(sock)) && '\0' != sockerrmsg[0])
 		{
 			pos = strlen(msg);
-			snprintf(&msg[pos], sizeof(msg) - pos, ";\n%s", sock->errormsg);
+			snprintf(&msg[pos], sizeof(msg) - pos, ";\n%s", sockerrmsg);
 		}
 		ermsg = msg;
 	}
@@ -1114,10 +1119,10 @@ StatementClass *SC_get_ancestor(StatementClass *stmt)
 {
 	StatementClass	*child = stmt, *parent;
 
-inolog("SC_get_ancestor in stmt=%x\n", stmt);
+inolog("SC_get_ancestor in stmt=%p\n", stmt);
 	for (child = stmt, parent = child->execute_parent; parent; child = parent, parent = child->execute_parent)
 	{
-		inolog("parent=%x\n", parent);
+		inolog("parent=%p\n", parent);
 	}
 	return child;
 }
@@ -1157,7 +1162,7 @@ SC_replace_error_with_res(StatementClass *self, int number, const char *message,
 	QResultClass	*self_res;
 	BOOL	repstate;
 
-inolog("SC_set_error_from_res %x->%x check=%d\n", from_res ,self, check);
+inolog("SC_set_error_from_res %p->%p check=%i\n", from_res ,self, check);
 	if (check)
 	{
 		if (0 == number)			return;
@@ -1202,7 +1207,7 @@ SC_error_copy(StatementClass *self, const StatementClass *from, BOOL check)
 	QResultClass	*self_res, *from_res;
 	BOOL	repstate;
 
-inolog("SC_error_copy %x->%x check=%d\n", from ,self, check);
+inolog("SC_error_copy %p->%p check=%i\n", from ,self, check);
 	if (self == from)	return;
 	if (check)
 	{
@@ -1250,7 +1255,7 @@ SC_full_error_copy(StatementClass *self, const StatementClass *from, BOOL allres
 {
 	PG_ErrorInfo		*pgerror;
 
-inolog("SC_full_error_copy %x->%x\n", from ,self);
+inolog("SC_full_error_copy %p->%p\n", from ,self);
 	if (self->__error_message)
 	{
 		free(self->__error_message);
@@ -1332,19 +1337,18 @@ SC_fetch(StatementClass *self)
 
 	Int2		num_cols,
 				lf;
-	Oid			type;
+	OID			type;
 	char	   *value;
 	ColumnInfoClass *coli;
 	BindInfoClass	*bookmark;
 
 	/* TupleField *tupleField; */
-	ConnInfo   *ci = &(SC_get_conn(self)->connInfo);
 
-inolog("%s statement=%x ommitted=0\n", func, self);
+inolog("%s statement=%p ommitted=0\n", func, self);
 	self->last_fetch_count = self->last_fetch_count_include_ommitted = 0;
 	coli = QR_get_fields(res);	/* the column info */
 
-	mylog("fetch_cursor=%d, %x->total_read=%d\n", SC_is_fetchcursor(self), res, res->num_total_read);
+	mylog("fetch_cursor=%d, %p->total_read=%d\n", SC_is_fetchcursor(self), res, res->num_total_read);
 
 	if (!SC_is_fetchcursor(self))
 	{
@@ -1359,7 +1363,7 @@ inolog("%s statement=%x ommitted=0\n", func, self);
 			return SQL_NO_DATA_FOUND;
 		}
 
-		mylog("**** SC_fetch: non-cursor_result\n");
+		mylog("**** %s: non-cursor_result\n", func);
 		(self->currTuple)++;
 	}
 	else
@@ -1368,7 +1372,7 @@ inolog("%s statement=%x ommitted=0\n", func, self);
 		retval = QR_next_tuple(res, self);
 		if (retval < 0)
 		{
-			mylog("**** SC_fetch: end_tuples\n");
+			mylog("**** %s: end_tuples\n", func);
 			if (QR_get_cursor(res) &&
 			    SQL_CURSOR_FORWARD_ONLY == self->options.cursor_type &&
 			    QR_once_reached_eof(res))
@@ -1379,20 +1383,30 @@ inolog("%s statement=%x ommitted=0\n", func, self);
 			(self->currTuple)++;	/* all is well */
 		else
 		{
-			mylog("SC_fetch: error\n");
-			SC_set_error(self, STMT_EXEC_ERROR, "Error fetching next row", func);
+			ConnectionClass *conn = SC_get_conn(self);
+
+			mylog("%s: error\n", func);
+			switch (conn->status)
+			{
+				case CONN_NOT_CONNECTED:
+				case CONN_DOWN:
+					SC_set_error(self, STMT_BAD_ERROR, "Error fetching next row", func);
+					break;
+				default:
+					SC_set_error(self, STMT_EXEC_ERROR, "Error fetching next row", func);
+			}
 			return SQL_ERROR;
 		}
 	}
 	if (QR_haskeyset(res))
 	{
-		Int4	kres_ridx;
+		SQLLEN	kres_ridx;
 
 		kres_ridx = GIdx2KResIdx(self->currTuple, self, res);
 		if (kres_ridx >= 0 && kres_ridx < res->num_cached_keys)
 		{
 			UWORD	pstatus = res->keyset[kres_ridx].status;
-inolog("SC_ pstatus[%d]=%x fetch_count=%d\n", kres_ridx, pstatus, self->last_fetch_count);
+inolog("SC_ pstatus[%d]=%hx fetch_count=" FORMAT_LEN "\n", kres_ridx, pstatus, self->last_fetch_count);
 			if (0 != (pstatus & (CURS_SELF_DELETING | CURS_SELF_DELETED)))
 				return SQL_SUCCESS_WITH_INFO;
 			if (SQL_ROW_DELETED != (pstatus & KEYSET_INFO_PUBLIC) &&
@@ -1414,7 +1428,7 @@ inolog("SC_ pstatus[%d]=%x fetch_count=%d\n", kres_ridx, pstatus, self->last_fet
 
 	result = SQL_SUCCESS;
 	self->last_fetch_count++;
-inolog("%s: stmt=%x ommitted++\n", func, self);
+inolog("%s: stmt=%p ommitted++\n", func, self);
 	self->last_fetch_count_include_ommitted++;
 
 	opts = SC_get_ARDF(self);
@@ -1427,9 +1441,9 @@ inolog("%s: stmt=%x ommitted++\n", func, self);
 	if ((bookmark = opts->bookmark) && bookmark->buffer)
 	{
 		char		buf[32];
-		UInt4	offset = opts->row_offset_ptr ? *opts->row_offset_ptr : 0;
+		SQLLEN	offset = opts->row_offset_ptr ? *opts->row_offset_ptr : 0;
 
-		sprintf(buf, "%ld", SC_get_bookmark(self));
+		sprintf(buf, FORMAT_ULEN, SC_get_bookmark(self));
 		SC_set_current_col(self, -1);
 		result = copy_and_convert_field(self, 0, buf,
 			 SQL_C_ULONG, bookmark->buffer + offset, 0,
@@ -1443,7 +1457,7 @@ inolog("%s: stmt=%x ommitted++\n", func, self);
 		extend_getdata_info(gdata, opts->allocated, TRUE);
 	for (lf = 0; lf < num_cols; lf++)
 	{
-		mylog("fetch: cols=%d, lf=%d, opts = %x, opts->bindings = %x, buffer[] = %x\n", num_cols, lf, opts, opts->bindings, opts->bindings[lf].buffer);
+		mylog("fetch: cols=%d, lf=%d, opts = %p, opts->bindings = %p, buffer[] = %p\n", num_cols, lf, opts, opts->bindings, opts->bindings[lf].buffer);
 
 		/* reset for SQLGetData */
 		gdata->gdata[lf].data_left = -1;
@@ -1461,7 +1475,7 @@ inolog("%s: stmt=%x ommitted++\n", func, self);
 				value = QR_get_value_backend(res, lf);
 			else
 			{
-				int	curt = GIdx2CacheIdx(self->currTuple, self, res);
+				SQLLEN	curt = GIdx2CacheIdx(self->currTuple, self, res);
 inolog("base=%d curr=%d st=%d\n", QR_get_rowstart_in_cache(res), self->currTuple, SC_get_rowset_start(self));
 inolog("curt=%d\n", curt);
 				value = QR_get_value_backend_row(res, curt, lf);
@@ -1531,6 +1545,7 @@ SC_execute(StatementClass *self)
 	ConnInfo   *ci;
 	UDWORD		qflag = 0;
 	BOOL		is_in_trans, issue_begin, has_out_para;
+	BOOL		use_extended_protocol;
 	int		func_cs_count = 0, i;
 
 	conn = SC_get_conn(self);
@@ -1548,6 +1563,7 @@ SC_execute(StatementClass *self)
 	 */
 #define	return	DONT_CALL_RETURN_FROM_HERE???
 	ENTER_INNER_CONN_CS(conn, func_cs_count);
+	oldstatus = conn->status;
 	if (CONN_EXECUTING == conn->status)
 	{
 		SC_set_error(self, STMT_SEQUENCE_ERROR, "Connection is already in use.", func);
@@ -1576,7 +1592,7 @@ SC_execute(StatementClass *self)
 	}
 	if (issue_begin)
 	{
-		mylog("   about to begin a transaction on statement = %x\n", self);
+		mylog("   about to begin a transaction on statement = %p\n", self);
 		if (PG_VERSION_GE(conn, 7.1))
 			qflag |= GO_INTO_TRANSACTION;
                 else if (!CC_begin(conn))
@@ -1586,7 +1602,6 @@ SC_execute(StatementClass *self)
                 }
 	}
 
-	oldstatus = conn->status;
 	/* self->status = STMT_EXECUTING; */
 	if (!SC_SetExecuting(self, TRUE))
 	{
@@ -1602,15 +1617,34 @@ SC_execute(StatementClass *self)
 	 * statement
 	 */
 	/* in copy_statement... */
+	use_extended_protocol = FALSE;
 	if (PREPARED_PERMANENTLY == self->prepared &&
 	    PROTOCOL_74(ci))
+		use_extended_protocol = TRUE;
+	else if (PREPARED_TEMPORARILY == self->prepared)
+	{
+		switch (SC_get_prepare_method(self))
+		{
+#ifndef	BYPASS_ONESHOT_PLAN_EXECUTION
+		case PARSE_TO_EXEC_ONCE:
+#endif /* BYPASS_ONESHOT_PLAN_EXECUTION */
+			case NAMED_PARSE_REQUEST:
+				use_extended_protocol = TRUE;
+		}
+		if (!use_extended_protocol)
+		{
+			SC_forget_unnamed(self);
+			SC_set_Result(self, NULL); /* discard the parsed information */
+		}
+	}
+	if (use_extended_protocol)
 	{
 		char	*plan_name = self->plan_name;
 
 		if (issue_begin)
 			CC_begin(conn);
 		res = SC_get_Result(self);
-inolog("get_Result=%x\n", res);
+inolog("get_Result=%p\n", res);
 		if (!plan_name)
 			plan_name = "";
 		if (!SendBindRequest(self, plan_name))
@@ -1638,7 +1672,7 @@ inolog("get_Result=%x\n", res);
 		char		fetch[128];
 		qflag |= (SQL_CONCUR_READ_ONLY != self->options.scroll_concurrency ? CREATE_KEYSET : 0); 
 
-		mylog("       Sending SELECT statement on stmt=%x, cursor_name='%s' qflag=%d,%d\n", self, SC_cursor_name(self), qflag, self->options.scroll_concurrency);
+		mylog("       Sending SELECT statement on stmt=%p, cursor_name='%s' qflag=%d,%d\n", self, SC_cursor_name(self), qflag, self->options.scroll_concurrency);
 
 		/* send the declare/select */
 		res = CC_send_query(conn, self->stmt_with_params, NULL, qflag, SC_get_ancestor(self));
@@ -1663,7 +1697,7 @@ inolog("get_Result=%x\n", res);
 			 * will correct for any discrepancies in sizes and adjust the
 			 * cache accordingly.
 			 */
-			sprintf(fetch, "fetch %d in \"%s\"", qi.row_size, SC_cursor_name(self));
+			sprintf(fetch, "fetch " FORMAT_LEN " in \"%s\"", qi.row_size, SC_cursor_name(self));
 
 			res = CC_send_query(conn, fetch, &qi, qflag, SC_get_ancestor(self));
 			if (SC_is_with_hold(self))
@@ -1674,7 +1708,7 @@ inolog("get_Result=%x\n", res);
 	else
 	{
 		/* not a SELECT statement so don't use a cursor */
-		mylog("      it's NOT a select statement: stmt=%x\n", self);
+		mylog("      it's NOT a select statement: stmt=%p\n", self);
 		res = CC_send_query(conn, self->stmt_with_params, NULL, qflag, SC_get_ancestor(self));
 
 		/*
@@ -1694,6 +1728,7 @@ inolog("get_Result=%x\n", res);
 				CC_commit(conn);
 		}
 	}
+	SC_forget_unnamed(self);
 
 	if (CONN_DOWN != conn->status)
 		conn->status = oldstatus;
@@ -1746,7 +1781,7 @@ inolog("get_Result=%x\n", res);
 				}
 			}
 
-inolog("!!%x->SC_is_concat_pre=%x res=%x\n", self, self->miscinfo, res);
+inolog("!!%p->SC_is_concat_pre=%x res=%p\n", self, self->miscinfo, res);
 			/*
 			 * special handling of result for keyset driven cursors. 			 * Use the columns info of the 1st query and
 			 * user the keyset info of the 2nd query.
@@ -1773,7 +1808,7 @@ inolog("!!%x->SC_is_concat_pre=%x res=%x\n", self, self->miscinfo, res);
 			else if (SC_is_concat_prepare_exec(self))
 			{
 				tres = res->next;
-inolog("res->next=%x\n", tres);
+inolog("res->next=%p\n", tres);
 				res->next = NULL;
 				if (res != SC_get_Result(self))
 					QR_Destructor(res);
@@ -1842,7 +1877,7 @@ inolog("!!SC_fetch return =%d\n", ret);
 		if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
 		{
 			APDFields	*apdopts = SC_get_APDF(self);
-			UInt4	offset = apdopts->param_offset_ptr ? *apdopts->param_offset_ptr : 0;
+			SQLULEN		offset = apdopts->param_offset_ptr ? *apdopts->param_offset_ptr : 0;
 			ARDFields	*ardopts = SC_get_ARDF(self);
 			const ParameterInfoClass	*apara;
 			const ParameterImplClass	*ipara;
@@ -1911,7 +1946,7 @@ int enqueueNeedDataCallback(StatementClass *stmt, NeedDataCallfunc func, void *d
 	stmt->callbacks[stmt->num_callbacks].data = data;
 	stmt->num_callbacks++;
 
-inolog("enqueueNeedDataCallack stmt=%x, func=%x, count=%d\n", stmt, func, stmt->num_callbacks);
+inolog("enqueueNeedDataCallack stmt=%p, func=%p, count=%d\n", stmt, func, stmt->num_callbacks);
 	return stmt->num_callbacks;
 }
 
@@ -1964,7 +1999,7 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 		QResultClass *res = SC_get_Result(self);
 		const ARDFields	*opts = SC_get_ARDF(self);
 		const APDFields	*apdopts = SC_get_APDF(self);
-		int	rowsetSize;
+		SQLLEN	rowsetSize;
 
 #if (ODBCVER >= 0x0300)
 		rowsetSize = (7 == self->transition_status ? opts->size_of_rowset_odbc2 : opts->size_of_rowset);
@@ -1982,10 +2017,10 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 		if (SC_get_errornumber(self) > 0)
 		{
 			qlog("                 ------------------------------------------------------------\n");
-			qlog("                 hdbc=%x, stmt=%x, result=%x\n", self->hdbc, self, res);
+			qlog("                 hdbc=%p, stmt=%p, result=%p\n", self->hdbc, self, res);
 			qlog("                 prepare=%d, internal=%d\n", self->prepare, self->internal);
-			qlog("                 bindings=%x, bindings_allocated=%d\n", opts->bindings, opts->allocated);
-			qlog("                 parameters=%x, parameters_allocated=%d\n", apdopts->parameters, apdopts->allocated);
+			qlog("                 bindings=%p, bindings_allocated=%d\n", opts->bindings, opts->allocated);
+			qlog("                 parameters=%p, parameters_allocated=%d\n", apdopts->parameters, apdopts->allocated);
 			qlog("                 statement_type=%d, statement='%s'\n", self->statement_type, nullcheck(self->statement));
 			qlog("                 stmt_with_params='%s'\n", nullcheck(self->stmt_with_params));
 			qlog("                 data_at_exec=%d, current_exec_param=%d, put_data=%d\n", self->data_at_exec, self->current_exec_param, self->put_data);
@@ -1997,7 +2032,7 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 
 			if (res)
 			{
-				qlog("                 fields=%x, backend_tuples=%x, tupleField=%d, conn=%x\n", res->fields, res->backend_tuples, res->tupleField, res->conn);
+				qlog("                 fields=%p, backend_tuples=%p, tupleField=%d, conn=%p\n", res->fields, res->backend_tuples, res->tupleField, res->conn);
 				qlog("                 fetch_count=%d, num_total_rows=%d, num_fields=%d, cursor='%s'\n", res->fetch_number, QR_get_num_total_tuples(res), res->num_fields, nullcheck(QR_get_cursor(res)));
 				qlog("                 message='%s', command='%s', notice='%s'\n", nullcheck(res->message), nullcheck(res->command), nullcheck(res->notice));
 				qlog("                 status=%d, inTuples=%d\n", QR_get_rstatus(res), QR_is_fetching_tuples(res));
@@ -2019,14 +2054,38 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
  *	Extended Query 
  */
 
+static BOOL
+RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
+{
+	BOOL	ret = TRUE;
+
+	if (SC_accessed_db(stmt))
+		return TRUE;
+	if (SQL_ERROR == SetStatementSvp(stmt))
+	{
+		char	emsg[128];
+
+		snprintf(emsg, sizeof(emsg), "internal savepoint error in %s", func);
+		SC_set_error(stmt, STMT_INTERNAL_ERROR, emsg, func);
+		return FALSE;
+	}
+	if (!CC_is_in_trans(conn) && !CC_is_in_autocommit(conn))
+	{
+		if (ret = CC_begin(conn), !ret)
+			return ret;
+	}
+	return ret;
+}
+
 BOOL
 SendBindRequest(StatementClass *stmt, const char *plan_name)
 {
 	CSTR	func = "SendBindRequest";
 	ConnectionClass	*conn = SC_get_conn(stmt);
-	SocketClass	*sock = conn->sock;
 
 	mylog("%s: plan_name=%s\n", func, plan_name);
+	if (!RequestStart(stmt, conn, func))
+		return FALSE;
 	if (!BuildBindRequest(stmt, plan_name))
 		return FALSE;
 
@@ -2049,17 +2108,11 @@ QResultClass *SendSyncAndReceive(StatementClass *stmt, QResultClass *res, const
 	IPDFields	*ipdopts;
 	QResultClass	*newres = NULL;
 
-	if (CC_is_in_trans(conn) && !SC_accessed_db(stmt))
-	{ 
-		if (SQL_ERROR == SetStatementSvp(stmt))
-		{
-			SC_set_error(stmt, STMT_INTERNAL_ERROR, "internal savepoint error in SendSynAndReceive", func);
-			return FALSE;
-		}
-	}
+	if (!RequestStart(stmt, conn, func))
+		return NULL;
 
-	SOCK_put_char(sock, 'S');	/* Sync message */
-	SOCK_put_int(sock, 4, 4);
+	SOCK_put_char(sock, 'S');	/* Sync command */
+	SOCK_put_int(sock, 4, 4);	/* length */
 	SOCK_flush_output(sock);
 
 	if (!res)
@@ -2094,6 +2147,16 @@ inolog(" response_length=%d\n", response_length);
 						mylog("%s: reached eof now\n", func);
 						QR_set_reached_eof(res);
 					}
+					else
+					{
+						int	ret1, ret2;
+
+						ret1 = ret2 = 0;
+						if (sscanf(msgbuffer, "%*s %d %d", &ret1, &ret2) > 1)
+							res->recent_processed_row_count = ret2;
+						else
+							res->recent_processed_row_count = ret1;
+					}
 				}
 				break;
 			case 'E': /* ErrorMessage */
@@ -2220,18 +2283,13 @@ SendParseRequest(StatementClass *stmt, const char *plan_name, const char *query)
 	CSTR	func = "SendParseRequest";
 	ConnectionClass	*conn = SC_get_conn(stmt);
 	SocketClass	*sock = conn->sock;
-	unsigned long	pileng, leng;
+	size_t		pileng, leng;
 
 	mylog("%s: plan_name=%s query=%s\n", func, plan_name, query);
-	if (CC_is_in_trans(conn) && !SC_accessed_db(stmt))
-	{ 
-		if (SQL_ERROR == SetStatementSvp(stmt))
-		{
-			SC_set_error(stmt, STMT_INTERNAL_ERROR, "internal savepoint error in SendParseRequest", func);
-			return FALSE;
-		}
-	}
-	SOCK_put_char(sock, 'P');
+	if (!RequestStart(stmt, conn, func))
+		return FALSE;
+
+	SOCK_put_char(sock, 'P'); /* Parse command */
 	if (SOCK_get_errcode(sock) != 0)
 	{
 		CC_set_error(conn, CONNECTION_COULD_NOT_SEND, "Could not send P request to backend", func);
@@ -2243,7 +2301,7 @@ SendParseRequest(StatementClass *stmt, const char *plan_name, const char *query)
 	if (!stmt->discard_output_params)
 		pileng += sizeof(UInt4) * (stmt->num_params - stmt->proc_return); 
 	leng = strlen(plan_name) + 1 + strlen(query) + 1 + pileng;
-	SOCK_put_int(sock, leng + 4, 4);
+	SOCK_put_int(sock, (Int4) (leng + 4), 4); /* length */
 inolog("parse leng=%d\n", leng);
 	SOCK_put_string(sock, plan_name);
 	SOCK_put_string(sock, query);
@@ -2272,32 +2330,27 @@ SendDescribeRequest(StatementClass *stmt, const char *plan_name)
 	CSTR	func = "SendDescribeRequest";
 	ConnectionClass	*conn = SC_get_conn(stmt);
 	SocketClass	*sock = conn->sock;
-	unsigned long	leng;
+	size_t		leng;
 	BOOL		sockerr = FALSE;
 
 	mylog("%s:plan_name=%s\n", func, plan_name);
-	if (CC_is_in_trans(conn) && !SC_accessed_db(stmt))
-	{
-		if (SQL_ERROR == SetStatementSvp(stmt))
-		{
-			SC_set_error(stmt, STMT_INTERNAL_ERROR, "internal savepoint error", func);
-			return FALSE;
-		}
-	}
-	SOCK_put_char(sock, 'D');
+	if (!RequestStart(stmt, conn, func))
+		return FALSE;
+
+	SOCK_put_char(sock, 'D'); /* Describe command */
 	if (SOCK_get_errcode(sock) != 0)
 		sockerr = TRUE;
 	if (!sockerr)
 	{
 		leng = 1 + strlen(plan_name) + 1;
-		SOCK_put_int(sock, leng + 4, 4);
+		SOCK_put_int(sock, (Int4) (leng + 4), 4); /* length */
 		if (SOCK_get_errcode(sock) != 0)
 			sockerr = TRUE;
 	}
 	if (!sockerr)
 	{
 inolog("describe leng=%d\n", leng);
-		SOCK_put_char(sock, 'S');
+		SOCK_put_char(sock, 'S'); /* describe a prepared statement */
 		if (SOCK_get_errcode(sock) != 0)
 			sockerr = TRUE;
 	}
@@ -2323,22 +2376,28 @@ SendExecuteRequest(StatementClass *stmt, const char *plan_name, UInt4 count)
 	CSTR	func = "SendExecuteRequest";
 	ConnectionClass	*conn;
 	SocketClass	*sock;
-	unsigned long	leng;
+	size_t		leng;
 
 	if (!stmt)	return FALSE;
 	if (conn = SC_get_conn(stmt), !conn)	return FALSE;
 	if (sock = conn->sock, !sock)	return FALSE;
 
 	mylog("%s: plan_name=%s count=%d\n", func, plan_name, count);
-	if (CC_is_in_trans(conn) && !SC_accessed_db(stmt))
+	if (!SC_is_fetchcursor(stmt))
 	{
-		if (SQL_ERROR == SetStatementSvp(stmt))
+		switch (stmt->prepared)
 		{
-			SC_set_error(stmt, STMT_INTERNAL_ERROR, "internal savepoint error", func);
-			return FALSE;
+			case NOT_YET_PREPARED:
+			case ONCE_DESCRIBED:
+				SC_set_error(stmt, STMT_EXEC_ERROR, "about to execute a non-prepared statement", func);
+				return FALSE;
 		}
 	}
-	SOCK_put_char(sock, 'E');
+	if (!RequestStart(stmt, conn, func))
+		return FALSE;
+
+	SOCK_put_char(sock, 'E'); /* Execute command */
+	SC_forget_unnamed(stmt); /* unnamed plans are unavailable */
 	if (SOCK_get_errcode(sock) != 0)
 	{
 		CC_set_error(conn, CONNECTION_COULD_NOT_SEND, "Could not send E Request to backend", func);
@@ -2347,13 +2406,13 @@ SendExecuteRequest(StatementClass *stmt, const char *plan_name, UInt4 count)
 	}
 
 	leng = strlen(plan_name) + 1 + 4;
-	SOCK_put_int(sock, leng + 4, 4);
+	SOCK_put_int(sock, (Int4) (leng + 4), 4); /* length */
 inolog("execute leng=%d\n", leng);
 	SOCK_put_string(sock, plan_name);
 	SOCK_put_int(sock, count, 4);
-	if (0 == count) /* Close message */
+	if (0 == count) /* will send a Close portal command */
 	{
-		SOCK_put_char(sock, 'C');
+		SOCK_put_char(sock, 'C');	/* Close command */
 		if (SOCK_get_errcode(sock) != 0)
 		{
 			CC_set_error(conn, CONNECTION_COULD_NOT_SEND, "Could not send C Request to backend", func);
@@ -2362,9 +2421,9 @@ inolog("execute leng=%d\n", leng);
 		}
 
 		leng = 1 + strlen(plan_name) + 1;
-		SOCK_put_int(sock, leng + 4, 4); /* Close portal */
+		SOCK_put_int(sock, (Int4) (leng + 4), 4); /* length */
 inolog("Close leng=%d\n", leng);
-		SOCK_put_char(sock, 'P');
+		SOCK_put_char(sock, 'P');	/* Portal */
 		SOCK_put_string(sock, plan_name);
 	}
 
@@ -2375,7 +2434,7 @@ BOOL	SendSyncRequest(ConnectionClass *conn)
 {
 	SocketClass	*sock = conn->sock;
 
-	SOCK_put_char(sock, 'S');	/* Sync message */
+	SOCK_put_char(sock, 'S');	/* Sync command */
 	SOCK_put_int(sock, 4, 4);
 	SOCK_flush_output(sock);
 
